<?php
/*$Id: nc_minishop.class.php 4487 2011-04-15 14:10:15Z denis $*/



class nc_minishop {
  // уведомление
  const NOTIFY_NONE = 0;  // нет
  const NOTIFY_ALERT = 1; // в сплывающем слое
  const NOTIFY_DIV = 2;   // через диалоговое окно

  // элемент "положить в корзину"
  const PUT_TEXT = 0;    // текст
  const PUT_IMG = 1;     // картинка
  const PUT_TEXTIMG = 2; // текст с картинкой
  const PUT_BUTTON = 3;  // кнопка
  const PUT_FORM = 4;    // кнопка с количесвтом

  // авторизация
  const AUTH_NONE = 0;    // не нужна
  const AUTH_SUGGEST = 1; // предлагать
  const AUTH_REQUIRE = 2; // требовать

  
	protected $cart;
	
	protected $db, $UI_CONFIG;
	protected $MODULE_FOLDER, $MODULE_PATH, $ADMIN_TEMPLATE;
	protected $settings;
	protected $TMPL;

	
	public function __construct() {
		
		@session_start(); 
    $this->cart = array();
		if (!empty($_SESSION['nc_minicart'])) {
			$this->cart = $_SESSION['nc_minicart'];
		} else {
			$_SESSION['nc_minicart'] = array();
		}
		
		// system superior object
		$nc_core = nc_Core::get_object();
		
		// global variables to internal
		$this->db = &$nc_core->db;
		$this->MODULE_FOLDER = $nc_core->MODULE_FOLDER;
		$this->MODULE_PATH = str_replace($nc_core->DOCUMENT_ROOT, "", $nc_core->MODULE_FOLDER)."minishop/";
		$this->TMPL = new nc_minishop_templates($this->MODULE_PATH);
		
		$this->settings = $nc_core->get_settings('','minishop');

	}

  static public function get_object () {
    return new self;
  }
	
	public function __get($name) {
		if ($name == 'settings' ) {
			return $this->settings;
		}
		if ($name == 'TMPL' ) {
			return $this->TMPL;
		}
	}

  /**
   * Функция генерации хэша товара по его имени и цене
   * @param array массив с ключами name и price
   * @return string хэш
   */
	public function generate_hash($params) {
    return md5($params['name'].$params['price'].substr(nc_Core::get_object()->get_settings('SecretKey'),10,15));
	}
	

  /**
   * Метод кладет товар в корзину
   * @param string имя товара
   * @param float цена товара
   * @param string хэш товара
   * @param float количество
   * @param string ссылка на товар
   * @return boolean результат операции
   */
	public function put_good($name,$price,$hash,$quantity = 1,$uri = '') {
		$result = false;
		if ( $name && $price && $this->generate_hash( array( 'name' => $name, 'price' => $price) ) == $hash ) {
			$params = array( 'name' => $name, 'price' => $price, 'hash' => $hash , 'quantity' => $quantity ? $quantity:1, 'uri' => $uri?$uri:'');
      $this->cart[] = $params;
		  $_SESSION['nc_minicart'][] = $params;
      return true;
		}
		return $result;
	}

  /**
   * Метод проверяет наличие товара в корзине, так же может поменять его количество
   * и удалить товар из корзины
   * @param string имя товара
   * @param float цена товара
   * @param float новое количество товара ( -1 - не менять, 0 - удлаить )
   * @return boolean
   */
	public function in_cart ($name,$price,$newvalue = -1) {
		$result = false;
		if (!empty($this->cart)) {
			foreach( $this->cart as $key => &$params ) {
				if ($params['name'] == $name && $params['price'] == $price) {
					if ( $newvalue == 0 ) { unset($this->cart[$key]); }
					if ( $newvalue > 0 ) { $params['quantity'] = $newvalue; }
					$_SESSION['nc_minicart'] = $this->cart;
					$result = true;
					break;
				}
			}
	   }
	   return $result;
	}


  /**
   * Функция возвращает html-код для помещения в коризну
   * или информер, что товар в корзине
   * @param string название товара
   * @param int цена товара
   * @param string ссылка на товар
   * @param int индетификатор товра в случае массового добавления, 0 - добавление одного товара
   * @param int 0 - принудительно показать кнопку добавления,
   *            1 - принудительно показать, что товар в корзине
   *            -1 - по наличию товара в корзине
   * @return string html-код
   */
	public function show_put_button($name, $price, $uri = '', $mass_id = 0, $incart = -1 ) {
    if ( $uri === 'this' ) $uri = $_SERVER['REQUEST_URI'];
    // ключ настройки с альтер.кнопкой
    $s_key = $mass_id ? 'mass_put_alternate' : 'put_button_alternate';
    $t_key = $mass_id ? 'template' : ($this->settings['put_button']+0);
    // товар не в корзине
		if (!$incart || ($incart == -1 && !$this->in_cart($name,$price))) {
			$template = $this->settings[$s_key] ?
                    $this->settings[$s_key] :
                    $this->TMPL->templates[ ($mass_id ? 'mass' : '').'put'][$t_key];

      $hash = $this->generate_hash( array('name' => $name, 'price' => $price));
			$name = rawurlencode($name);
			$price = rawurlencode($price);
			$uri = rawurlencode($uri);
			$id = $mass_id ? $mass_id : 1;
		}
    if ( $incart == 1 || ($incart == -1 && $this->in_cart($name,$price)) ) { // товар уже в корзине
      $template = $this->settings['already_in_cart_alternate'] ?
                    $this->settings['already_in_cart_alternate'] :
                    $this->TMPL->templates['incart'][ $this->settings['already_in_cart']+0 ];
		}

    eval("\$result = \"".$template."\";");
		return $result;
	}

  /**
   * Метод возвращает "хэдер"  области массового добавления товаров
   * @return string html-код
   */
  public function mass_put_header () {
    $template = $this->TMPL->templates['massput']['header'];
    eval("\$result = \"".$template."\";");
    return $result;
  }

  /**
   * Метод возвращает "футер" области массового добавления товаров
   * @return string html-код
   */
  public function mass_put_footer () {
    $template = $this->TMPL->templates['massput']['button'];
    eval("\$result = \"".$template."\";");
    return $result;
  }

  /**
   * Метод возвращает html-код, который используется для нотификации
   * @return string html-код
   */
  public function get_notify ( ) {
    $nc_core = nc_Core::get_object();
    if ( $this->settings['notify'] == self::NOTIFY_NONE ) {
      return array( 'type' => $this->settings['notify'], 'text' => '' );
    }
    $key = $this->settings['notify'] == self::NOTIFY_ALERT ? 'notify_alert' : 'notify_div';
    $template = $this->settings[$key];
    if ( !$template ) {
      $template = $this->TMPL->templates['notify'][ $this->settings['notify'] ];
    }

    $carturl =  $this->cart_url();
    $orderurl = $this->addorder_url();

    eval("\$result = \"".$template."\";");

    if ( $this->settings['notify'] == self::NOTIFY_DIV ) {
      $result = "<div id='nc_mslayer' style='display:none;'>".$result."<span class='simplemodal-close'></span></div>
<script src='".$nc_core->SUB_FOLDER.$nc_core->HTTP_ROOT_PATH."require/js/jquery.simplemodal-1.3.3.min.js' type='text/javascript'></script>
<script type='text/javascript'>
  $('#nc_mslayer').modal();
</script>";
    }

    return array( 'type' => $this->settings['notify'], 'text' => $result );
  }

  /**
   * Количество товаров в корзине
   * @return int количество товаров
   */
  public function cart_count ( ) {
    return count($this->cart);
  }

  /**
   * Содержимое корзины
   * @return array двухмерный массив, ключи: name, price, quantity
   */
  public function cart_content () {
    return $this->cart;
  }

  /**
   * Сумма товаров в корзине
   * @param bool учитывать скидки или нет
   * @return float сумма товаров
   */
  public function cart_sum ( $with_discount = 0 ) {
    $sum = 0;
    foreach ( $this->cart as $v ) {
      $sum += $v['price']*$v['quantity'];
    }
    if ( $with_discount ) $sum = (100 - $this->suitable_discount())*$sum/100;
    return $sum;
  }


  /**
   * Очистка корзины
   */
  public function cart_clear () {
    $this->cart = $_SESSION['nc_minicart'] = array();
  }


  /**
   * Ссылка на корзину
   * @return string ссылка
   */
  public function cart_url ( ) {
    $url = $this->settings['cart_url'];
    if ( !$url ) {
      $nc_core = nc_Core::get_object();
      $info = $nc_core->db->get_row("SELECT s.`Hidden_URL` AS `url`, sc.`Sub_Class_ID` AS `cc`
        FROM `Subdivision` AS `s`, `Sub_Class` AS `sc`, `Class` as `c`
        WHERE `c`.`Class_ID` = '".intval($this->settings['cart_class_id'])."'
        AND sc.`Class_ID` = c.`Class_ID`
        AND sc.`Subdivision_ID` = s.`Subdivision_ID` LIMIT 1 ", ARRAY_A);
      $url = $info['url'];
      $nc_core->set_settings('cart_url', $url, 'minishop');
      $nc_core->set_settings('cart_cc', $info['cc'], 'minishop');
      $this->settings = $nc_core->get_settings('', 'minishop');
    }

    return $url;
  }


  public function addorder_url () {
    $url = $this->settings['addorder_url'];
    return $url;
  }

  public function order_url () {
    $url = $this->settings['order_url'];
    return $url;
  }


  public function cart_class_id () {
    return $this->settings['cart_class_id'];
  }

  public function order_class_id () {
    return $this->settings['order_class_id'];
  }


  public function order_addfrom ( ) {
     $component = new nc_Component ( $this->order_class_id () );
     $info = nc_Core::get_object()->sub_class->get_by_id( $this->settings['order_cc'] );
     return $component->add_form ( $info['Catalogue_ID'], $info['Subdivision_ID'], $info['Sub_Class_ID'], 1);
  }

  /**
   * Метод возвращает html-код состояния корзины
   * @return string html-код
   */
	public function show_cart_state () {
    if ( empty($this->cart )) {
      $template = $this->settings['cart_empty'] ?
                    $this->settings['cart_empty'] :
                    $this->TMPL->templates['cart']['empty'];
    }
    else {
      $cartcount = count($this->cart);
      $cartsum = $this->cart_sum(1);
      $cartcurrency = $this->get_currency();
      $carturl =  $this->cart_url();
      $orderurl = $this->addorder_url();

      $template = $this->settings['cart_full'] ?
                    $this->settings['cart_full'] :
                    $this->TMPL->templates['cart']['nonempty'];
    }
    
    eval("\$result = \"".$template."\";");
    return "<div id='nc_minishop_cart'>".$result."</div>";
	}


  /**
   * Валюта магазина
   * @return <type>
   */
  public function get_currency () {
    // $cartcurrency = $this->settings['UseCurrency'] == '1' ? $this->settings['AltCurrencyName'] : NETCAT_MODULE_MINISHOP_CURRENCY_RUB ;
    return NETCAT_MODULE_MINISHOP_CURRENCY_RUB;
  }


  /**
   * Метод ищет подходящую скидку под заданную стоимость.
   * Если стоимость не задана - то берется сумма из корзины
   * @param float сумма заказа
   * @return float скидка в процентах 
   */
  public function suitable_discount ( $sum = 0 ) {
    if ( !$this->settings['discount_enabled'] ) return 0;
    if ( !$sum ) $sum = $this->cart_sum ();

    $d = array();
    $d_str = $this->settings['discounts'];
    if ( $d_str) $d = unserialize ($d_str);
    foreach ( $d as $v ) {
      if ( $v['from'] <= $sum && $sum <= $v['to'] ) return $v['value'];
    }
    return 0;
  }

  public function save_order ( $order_id ) {
    $order = new nc_minishop_order($order_id);
    $order->put_goods( $this->cart_content() );
    $order->apply_discount( $this->suitable_discount() );

    $this->cart_clear();
  }


  public function edit_order ($order_id,$goods) {
    $order = new nc_minishop_order($order_id);
    $order->put_goods( $goods, 0 );
  }


  public function get_mail ( $to, $order_id ) {
    $order = new nc_minishop_order($order_id);
    $useremail = $order->get('Email');

    // позиции заказа
    $content = '';
    foreach ( $order->content() as $v ) {
      $content .= $v['name']." - ".$v['quantity'].NETCAT_MODULE_MINISHOP_PIECES." - ".$v['price']." ".$this->get_currency()."\r\n<br/>";
    }

    $macro = array('SHOP_NAME' => $this->settings['shopname'],
                   'SITE_URL' => $_SERVER['HTTP_HOST'],
                   'ORDER_NUM' => $order_id,
                   'FINAL_COST' => $order->get('FinalCost'),
                   'USER_NAME' => $order->get('Name'),
                   'CONTENT' => $content,
                   'DISCOUNT' => $order->get('Discount'));
    // шаблоны писем
    $subject = $this->settings['mail_subject_'.$to];
    $body = $this->settings['mail_body_'.$to];
    $ishtml = $this->settings['mail_ishtml_'.$to];
    // замена макропеременных
    foreach ( $macro as $k => $v ) {
      $subject = str_replace ('%'.$k, $v, $subject);
      $body = str_replace ('%'.$k, $v, $body);
    }

    return array('subject' => $subject, 'body' => $body, 'html' => $is_hrml );
  }


  public function get_afterorder_text ( $order_id ) {
    $template = $this->settings['orderfrom_text'];
    eval("\$result = \"".$template."\";");
    return $result;
  }


  public function  __destruct() {
    // пересчет корзины, чтобы индексы шли по порядку (0,1,2...)
    $tmp = array();
    if ( $this->cart ) foreach ( $this->cart as $v ) $tmp[] = $v;
    $_SESSION['nc_minicart'] = $tmp;

  }


  

} 



?>
