<?php
/* $Id: admin.inc.php 4606 2011-05-09 21:41:38Z denis $ */

/**
  * Функции и классы, используемые в административном интерфейсе
  *  (netcat/admin, netcat - режим редактирования).
  */

/**
  * Конвертирует массив в JSON
  * @param array
  * @return string
  */
function nc_array_json($array) {
  // isn't it an associative array? [guess]
  $array_keys = array_keys($array);
  sort($array_keys);
  $array_is_numeric = ($array_keys[0]=="0" ? true : false);

  $ret_values = array();
  foreach ($array as $k => $v) {
    if (is_object($v)) { continue; } // ВНУТРЕННИЕ ОБЪЕКТЫ, тонкое место
    elseif (is_array($v)) { $v = nc_array_json($v); }
    elseif (is_bool($v)) { $v = $v ? 'true' : 'false'; }
    elseif (!is_numeric($v)) { $v = "\"".addcslashes($v, '"')."\""; }

    if ($array_is_numeric) {
     $ret_values[] = $v;
    }
    else {
     $ret_values[] = (is_numeric($k) ? $k : "\"".addcslashes($k, '"')."\"") . ": " . $v;
    }
  }

  if ($array_is_numeric) { return "[" . join(", ",  $ret_values) . "]"; }

  return "{" . join(", ",  $ret_values) . "}";
}

/**
 * Вывод системного сообщения
 *
 * @param string $message системное сообщение
 * @param string $status тип системного сообщения
 * @param array $array массив параметров системного сообщения
 *
 * @return bool всегда возвращает true
 */
function nc_print_status($message, $status, $array = '') {
  if ($array) {
    vprintf("<div id='statusMessage' class='status_".$status."'>".$message."</div>\n", $array);
  } else {
    print "<div id='statusMessage' class='status_".$status."'>".$message."</div>\n";
  }
  return true;
}
/**
  * Класс, выводящий состояние интерфейса для внешнего фрейма в виде json
  */
class ui_config {
  /**
    * описание см. mainView.currentSettings
    */
  var $headerText;
  var $headerImage;
  var $tabs = array();
  var $toolbar = array();
  var $actionButtons = array();
  var $activeTab;
  var $activeToolbarButtons;
  var $treeMode;
  var $treeSelectedNode;
  var $treeChanges;
  var $locationHash;

  /**
    * Конструктор
    * @param array параметры (ключ-значение)
    */
  function ui_config($config_array=array()) {
    foreach ($config_array as $k=>$v) {
      $this->$k = $v;
    }
  }

  /**
    * Вывод настроек в формате json виде <script>parent.mainView.updateSettings()</script>
    */
  function to_json() {
    $ret = get_object_vars($this);
    $ret['tabsCrc'] = crc32(serialize($ret['tabs']));
    $ret['toolbarCrc'] = crc32(serialize($ret['toolbar']));
    $ret['actionButtonsCrc'] = crc32(serialize($ret['actionButtons']));
    return "<script>if (parent && parent.mainView) { parent.mainView.updateSettings(".nc_array_json($ret)."); }</script>\n";
  }
} // of class ui_config


/**
  * для совместной работы с ui_config_subdivision, ...
  */
class ui_subdivision_data {
  var $catalogue_id;
  var $subdivision_id;
  var $subdivision_name;
  var $subdivision_url;
  var $subdivision_checked;

  // private
  var $subclasses = array();
  var $moderated_subclasses = array();
  var $administered_subclasses = array();
  var $is_subdivision_admin = false;

  /**
    * загрузить данные о разделе по id раздела
    * @param integer
    */
  function fetch_by_subdivision_id($sub_id) {
    settype($sub_id, 'integer');
    if (!$sub_id) return;

    $qry = "SELECT sc.Sub_Class_ID,
                   sc.Sub_Class_Name,
                   sc.Subdivision_ID,
                   sc.Class_ID,
                   sd.Subdivision_Name,
                   sd.Catalogue_ID,
                   sd.Hidden_URL,
                   sd.Checked
              FROM Sub_Class as sc,
                   Subdivision as sd
             WHERE sc.Subdivision_ID = $sub_id
               AND sc.Subdivision_ID = sd.Subdivision_ID
             ORDER BY sc.Priority";
    $this->fetch_by_query($qry);

    // если нет шаблонов в разделе
    if (!$this->subdivision_id) {
      global $db;
      $sub_data = $db->get_row("SELECT Subdivision_ID, Subdivision_Name, Catalogue_ID, Hidden_URL, Checked
                                  FROM Subdivision
                                 WHERE Subdivision_ID = $sub_id", ARRAY_A);

      $this->catalogue_id = $sub_data['Catalogue_ID'];
      $this->subdivision_id = $sub_data['Subdivision_ID'];
      $this->subdivision_name = $sub_data['Subdivision_Name'];
      $this->subdivision_url = $sub_data['Hidden_URL'];
      $this->subdivision_checked = $sub_data['Checked'];

      $this->check_rights();
    }
  }

  /**
    * загрузить данные о разделе по id шаблона в разделе
    * @param integer
    */
  function fetch_by_subclass_id($cc_id) {
    settype($cc_id, 'integer');
    $qry = "SELECT sc.Sub_Class_ID,
                   sc.Sub_Class_Name,
                   sc.Subdivision_ID,
                   sc.Class_ID,
                   sd.Subdivision_Name,
                   sd.Catalogue_ID,
                   sd.Hidden_URL,
                   sd.Checked
              FROM Sub_Class as sc,
                   Sub_Class as ref,
                   Subdivision as sd
             WHERE ref.Sub_Class_ID = $cc_id
               AND ref.Subdivision_ID = sc.Subdivision_ID
               AND sc.Subdivision_ID = sd.Subdivision_ID
             ORDER BY sc.Priority";
    $this->fetch_by_query($qry);
  }

  /**
    * 'собственно функция для получения нужной информации'
    * @access private
    */
  function fetch_by_query($qry) {

    global $db;
    $this->subclasses = $db->get_results($qry, ARRAY_A);

    if ($this->subclasses[0]) {
      $this->catalogue_id = $this->subclasses[0]['Catalogue_ID'];
      $this->subdivision_id = $this->subclasses[0]['Subdivision_ID'];
      $this->subdivision_name = $this->subclasses[0]['Subdivision_Name'];
      $this->subdivision_url = $this->subclasses[0]['Hidden_URL'];
      $this->subdivision_checked = $this->subclasses[0]['Checked'];

      $this->check_rights();
    }
  }

  /**
    * Fills $administered_subclasses, $moderated_subclasses
    * @access private
    */
  function check_rights() {

    global $AUTH_USER_ID;
    if ( !$AUTH_USER_ID ) {
      Refuse();
      exit;
    }

    global $perm;

    // has rights for everything in the sub
    $user_is_cool = $perm->isSupervisor() || $perm->isGuest() ;


    if ($user_is_cool) {
      $this->moderated_subclasses = &$this->subclasses;
      $this->administered_subclasses = &$this->subclasses;
      $this->is_subdivision_admin = true;
      return true;
    }
    if ($perm->isSubdivisionAdmin($this->subdivision_id) ) {
      // $this->administered_subclasses = &$this->subclasses;
      $this->is_subdivision_admin = true; 
    }
    
     if ($perm->isSubdivision($this->subdivision_id, 16) ) {
      ;// $this->moderated_subclasses = &$this->subclasses;
    }
    
    
    if (!sizeof($this->subclasses)) return;

    // subclass admin / moderator
    foreach ($this->subclasses as $idx=>$sc) { 
      if ($perm->isSubclassAdmin($sc['Sub_Class_ID'])) { 
        $this->administered_subclasses[] = &$this->subclasses[$idx];
        //$this->moderated_subclasses[] = &$this->subclasses[$idx];
      }
      if ($perm->isSubclass($sc['Sub_Class_ID'], 16)) {
        $this->moderated_subclasses[] = &$this->subclasses[$idx];
      }
    }

    // subdivision moderator
    $user_is_moderator = ($perm->isCatalogue($this->catalogue_id, 16) ||
                          $perm->isSubdivision($this->subdivision_id, 16));

    if ($user_is_moderator) {
      $this->moderated_subclasses = &$this->subclasses;
    }

  }

  /**
    * является ли пользователь администратором данного раздела
    * @return boolean
    */
  function is_subdivision_admin() {
    return $this->is_subdivision_admin;
  }

  /**
    * является ли пользователь модератором шаблона $cc_id или если $cc_id не задан
    * является ли пользователь модератором хотя бы одного шаблона в разделе
    * @access public
    * @return boolean
    */
  function is_subclass_moderator($cc_id=0) {
    if (!$cc_id) return (sizeof($this->moderated_subclasses) ? true : false);

    foreach ((array)$this->moderated_subclasses as $sc) {
      if ($sc["Sub_Class_ID"] == $cc_id) return true;
    }
    return false;
  }

  /**
    * является ли пользователь администратором [$cc_id|хотя бы одного] шаблона в разделе
    * @access public
    * @return boolean
    */
  function is_subclass_admin($cc_id=0) {
    return (sizeof($this->administered_subclasses) ? true : false);

    foreach ((array)$this->administered_subclasses as $sc) {
      if ($sc["Sub_Class_ID"] == $cc_id) return true;
    }
    return false;
  }

  /**
    * массив с информацией о всех шаблонах-в-разделе, которые пользователь
    * может модерировать (в т.ч. те, которые может администрировать)
    * @access public
    * @return array
    */
  function get_moderated_subclasses() {
    return $this->moderated_subclasses;

  }

  /**
    * массив с информацией о всех шаблонах-в-разделе, которые пользователь
    * может администрировать
    * @access public
    * @return array
    */
  function get_administered_subclasses() {
    return $this->administered_subclasses;
  }

  /**
    * получить id первого шаблона в разделе, который юзер администрирует
    */
  function get_first_administered_subclass_id() {
    return $this->administered_subclasses[0]['Sub_Class_ID'];
  }

  /**
    * получить id первого шаблона в разделе, который юзер модерирует
    */
  function get_first_moderated_subclass_id() {
    return $this->moderated_subclasses[0]['Sub_Class_ID'];
  }

}

/**
  * общий класс для работы с разделом, шаблонами и объектами в нем
  * отвечает за то, чтобы создать правильный (соответствующий
  * полномочиям пользователя) набор вкладок
  */
class ui_config_subdivision_gereric extends ui_config {
  /**
    * объект с информацией о разделе и шаблонах в нем (ui_subdivision_data)
    */
  var $sub;

  /**
    * @param integer
    * @param integer
    */
  function init($sub_id, $cc_id=0) {
    global $perm;
    $this->sub = new ui_subdivision_data();
    if ($sub_id) { $this->sub->fetch_by_subdivision_id($sub_id); }
    elseif ($cc_id) { $this->sub->fetch_by_subclass_id($cc_id); }
    else { trigger_error("Wrong parameters for ui_config_subdivision_gereric::init", E_USER_ERROR); }

    $this->headerText = $subdivision["Subdivision_Name"];
    $this->headerImage = 'i_folder_big.gif';

    // определим набор вкладок
    $this->tabs = array();
    
    // 1. Информация - только админ раздела и выше
    if ($this->sub->is_subdivision_admin() ) {
      $this->tabs[] = array('id' => 'info',
                            'caption' => STRUCTURE_TAB_INFO,
                            'location' => "subdivision.info({$this->sub->subdivision_id})"
                           );  
    }
    
    
    // 2. Настройки - только админ раздеа и выше
    if ($this->sub->is_subdivision_admin() ) {
         $this->tabs[] = array('id' => 'settings',
                            'caption' => STRUCTURE_TAB_SETTINGS,
                            'location' => "subdivision.edit({$this->sub->subdivision_id})"
                           );  
    }
    
    
    // 3. Используемые компонеты. Админ раздела видит все. Админ сс - только свои.
    if ( $this->sub->is_subdivision_admin() ) {//|| $this->sub->is_subclass_admin() )
      $loc = ($this->sub->subclasses) ? "subclass.list({$this->sub->subdivision_id})" : "subclass.add({$this->sub->subdivision_id})";
      $this->tabs[] = array('id' => 'subclass',
                            'caption' => STRUCTURE_TAB_USED_SUBCLASSES,
                            'location' => $loc
                           );
    }
    else if ( $this->sub->is_subclass_admin() ) { // admin только сабкласса
      $active_cc_id =  $this->sub->get_first_administered_subclass_id();
      $this->tabs[] = array('id' => 'subclass',
                            'caption' => STRUCTURE_TAB_USED_SUBCLASSES,
                            'location' => "subclass.edit(".$active_cc_id.", ".$this->sub->subdivision_id.")"
                           );
        
    }
    

    // 4. Редактирование
    $active_cc_id = $this->sub->get_first_moderated_subclass_id();
    //print $active_cc_id;
    if ( $active_cc_id ) {
                   $this->tabs[] = array('id' => 'objects',
                          'caption' => STRUCTURE_TAB_EDIT,
                          'location' => "object.list($active_cc_id)"
                         );
    }

    // 5. Просмотр
    $this->tabs[] = array('id' => 'view',
                          'caption' => STRUCTURE_TAB_PREVIEW,
                          'location' => $active_cc_id ? "subclass.view(0,$active_cc_id)"
                                                      : "subdivision.view({$this->sub->subdivision_id},0)"
                         );

    // Прочие общие настройки
    $this->headerImage = 'i_folder_big.gif';
    $this->headerText = $this->sub->subdivision_name;
    $this->treeMode = 'sitemap';
    $this->treeSelectedNode = "sub-". $this->sub->subdivision_id;
  }

}

class ui_config_objects extends ui_config_subdivision_gereric {

  function ui_config_objects($cc_id) {
    $this->init(0, $cc_id);
    if (!$cc_id) { trigger_error("Wrong cc_id in ui_config_objects::ui_config_objects", E_USER_ERROR); }

    $this->activeTab = 'objects';

    // кнопки
    $subclasses = $this->sub->get_moderated_subclasses();

    if (sizeof($subclasses) > 1) {
      $enable_subclass_drag = $this->sub->is_subdivision_admin();
      foreach ($subclasses as $sc) {
        $this->toolbar[] = array('id' => "subclass$sc[Class_ID]-$sc[Sub_Class_ID]",
                                 'caption' => $sc["Sub_Class_Name"],
                                 'location' => "object.list($sc[Sub_Class_ID])",
                                 'group' => "grp1",
                                 'acceptDropFn' => 'subclassAcceptDrop',
                                 'onDropFn' => 'subclassOnDrop',
                                 'dragEnabled' => $enable_subclass_drag,
                                 'metadata' => array("subdivisionId"=>$sc['Subdivision_ID'])
                                );
        $cc_subs[$sc["Sub_Class_ID"]] = $sc["Class_ID"];
      }

      $this->activeToolbarButtons[] = "subclass{$cc_subs[$cc_id]}-$cc_id";
    }

  }

}

/**
  * Инструменты
  */
class ui_config_tool extends ui_config {

  /**
    * @param string имя тулзы
    * @param string файл с картинкой для заголовка
    * @param string адрес (#хэш)
    */
  function ui_config_tool($name, $caption, $big_icon, $location) {
    $this->headerText = $name;
    $this->headerImage = $big_icon;
    $this->tabs = array(array("id"=>"tool", "caption"=>$caption, "location"=>$location));
    $this->activeTab = "tool";
    $this->locationHash = $location;
  }
}

class ui_config_trash extends ui_config {

  /**
    * @param string name - текст над вкладками
    * @param string page - активная вкладка
    * @param string caption1 текст первой вкладки
    * @param string caption2 текст второй вкладки
    */
  function ui_config_trash($name, $page, $caption1) {
    $this->headerText = $name;
    $this->headerImage = 'i_tool_trash_big.gif';
    $this->tabs = array(
        array("id"=>"trashlist", "caption"=>$caption1, "location"=>'#trash.list')
        //array("id"=>"trashsettings", "caption"=>$caption2, "location"=>'#trash.settings')
     );
    $this->activeTab =  $page == 'settings' ? "trashsettings" : "trashlist";
    $this->locationHash = '#trash.'.( $page == 'settings' ? "settings" : "list");
  }
}


/**
  * Функция обработки URL и форм для редактирования объектов внутри netcat-admin.
  * Использование: ob_start("nc_postprocess_admin_page").
  * Эффекты:
  * - Добавляет inside_iframe к ссылкам для редактирования и в формы
  *   (т.о. модификация альтернативных форм версии 2.4 не требуется)
  * @return string результат
  */
function nc_postprocess_admin_page($str) {
  if (!nc_strlen($str)) return "";

  global $inside_admin, $HTTP_ROOT_PATH, $SUB_FOLDER;
  if (!$inside_admin) return $str;
  // current adress
  global $parsed_url;

	$local_links_re = "(                                                  # $1: whole link
		(                               			                              # $2: script path
			(?:                           			                              # group: absolute path
				(?:                                                             #
					".preg_quote($parsed_url['scheme']).":\/\/                    # protocol
					".preg_quote(str_replace("www.", "", $parsed_url['host']))."  # host
					(?::".preg_quote($parsed_url['port']).")? 	                  # port
				)?                         			                                # host part is optional
				".preg_quote($SUB_FOLDER.$HTTP_ROOT_PATH)." 	                  # /netcat/
				(?![a-z]+\/)                 			                              # but not /netcat/dirname/
			)                             			                              # optional (relative path)
			(?:[^\/\?\#'\"\s>]*)?        			                                # script name (optional)
		)                               			                              # grab path
		(?:\?([^'\"\s>]+))?            				                              # followed by GET params ($3)
	)";

  // look for forms and hrefs
  nc_preg_match_all("@
    <(?:a|form) \s
    [^>]*
    (?:href|action)\s*=\s*[\"']?".$local_links_re."[\"']?
    [^>]*
    >
    @xs", $str, $regs, PREG_PATTERN_ORDER);

  // regs[1] - full link; [2] path; [3] get params

  foreach ($regs[3] as $num=>$get_params) {
    if (!$get_params || !preg_match("/[?&]inside_admin=/", $get_params)) {
      $new_link = str_replace($regs[1][$num],
                              $regs[2][$num] . "?inside_admin=".$inside_admin.($get_params ? "&amp;".$get_params : ""),
                              $regs[0][$num]);
      $str = str_replace($regs[0][$num], $new_link, $str);
    }
  }

  return $str;
}


/**
  * Перемещение объекта из одного шаблона в разделе в другой.
  *
  * Пользователь должен обладать правами: изменение в разделе, где
  * находится объект, и удаление в разделе, куда переносится объект.
  *
  * @param integer ID класса объекта
  * @param integer ID объекта
  * @param integer ID шаблона в разделе, куда переносится объект
  * @return boolean
  */
function nc_move_message($class_id, $message_id, $destination_cc_id) {
  global $nc_core;
  
  $class_id = (int)$class_id;
  $message_id = (int)$message_id;
  $destination_cc_id = (int)$destination_cc_id;

  if (!$class_id || !$message_id || !$destination_cc_id) {
    trigger_error("Wrong parameters for nc_move_message()", E_USER_WARNING);
    return false;
  }

  $db = $nc_core->db; 

  // перемещаемое сообщение
  $message = $db->get_row("SELECT sd.`Catalogue_ID`,
                                  m.*
                             FROM `Message".$class_id."` AS m,
                                  `Subdivision` as sd
                            WHERE m.`Message_ID`='".$message_id."'
                              AND m.`Subdivision_ID`=sd.`Subdivision_ID` ", ARRAY_A);

  // сабкласс назначения
  $dest_subclass = $db->get_row("SELECT sd.`Catalogue_ID`,
                                        sc.`Sub_Class_ID`,
                                        sc.`Subdivision_ID`,
                                        sc.`Class_ID`,
                                        IFNULL(MAX(m.`Priority`)+1,1) as Next_Priority
                                   FROM (`Sub_Class` as sc,
                                        `Subdivision` as sd)
                                        LEFT JOIN `Message".$class_id."` as m
                                          ON m.`Sub_Class_ID`=sc.`Sub_Class_ID`
                                  WHERE sc.`Sub_Class_ID`='".$destination_cc_id."'
                                    AND sc.`Subdivision_ID`=sd.`Subdivision_ID`
                                  GROUP BY m.`Sub_Class_ID` ", ARRAY_A);

  // существует ли объект и компонент в разделе
  if (!$message || !$dest_subclass) {
    $what = ($message ? 'subclass' : 'object');
    trigger_error("nc_move_message: $what doesn't exist", E_USER_WARNING);
    return false;
  }
  
  // перемещать можно только в рамках одного компонента
  if ($dest_subclass['Class_ID'] != $class_id) {
    trigger_error("nc_move_message: destination subclass belongs to different class", E_USER_WARNING);
    return false;
  }
  
  // перенос в самого себя
  if ($dest_subclass['Sub_Class_ID'] == $message['Sub_Class_ID']) { return true; } 

  // права
  global $perm;
  $has_rights = false;
  $has_rights = ($perm->isSubClass($message['Sub_Class_ID'], 48) && $perm->isSubClass($dest_subclass['Sub_Class_ID'], 48) );
  if (!$has_rights) {
    trigger_error("nc_move_message: insufficient rights", E_USER_WARNING);
    return false;
  }

  // файлы
  $file_fields = $db->get_results("SELECT `Field_ID`, `Field_Name` FROM `Field` WHERE `Class_ID`='".$class_id."' AND TypeOfData_ID = '6' ", ARRAY_A);
  if ( !empty($file_fields)) {
    $FILES_FOLDER = $nc_core->FILES_FOLDER;
    $DOCUMENT_ROOT = $nc_core->DOCUMENT_ROOT;
    $SUB_FOLDER = $nc_core->SUB_FOLDER;   
    $DIRCHMOD = $nc_core->DIRCHMOD;
    $INCLUDE_FOLDER = $nc_core->INCLUDE_FOLDER;
    
    // создание директорий
    $dirs = array($FILES_FOLDER . $dest_subclass['Subdivision_ID'],
                  $FILES_FOLDER . $dest_subclass['Subdivision_ID'] . '/' . $dest_subclass['Sub_Class_ID']);

    foreach ($dirs as $dir) {
      if (!file_exists($dir) && !mkdir($dir, $DIRCHMOD)) { return false; } // can't create dir
      @chmod($dir, $DIRCHMOD);
    }
    unset($dirs);

    $dest_path = "/$dest_subclass[Subdivision_ID]/$dest_subclass[Sub_Class_ID]/";

    require_once($INCLUDE_FOLDER."s_common.inc.php");
    
    foreach ($file_fields as $field) {
      $value = $message[$field['Field_Name']];
      if ( !$value ) continue; // файла нет
      
      $file_id = $db->get_var("SELECT `ID` FROM `Filetable` WHERE `Field_ID` = '".$field['Field_ID']."' AND `Message_ID` = '".$message_id."'");
      if ( $file_id ) {
        $file_path = nc_file_path($class_id, $message_id, $field['Field_ID']);
        if ( $file_path ) preg_match("#/([0-9a-f]{32})$#i", $file_path, $regs);
        if (file_exists(rtrim($DOCUMENT_ROOT, '/') . $SUB_FOLDER . $file_path) && !rename($DOCUMENT_ROOT.$SUB_FOLDER.$file_path, rtrim($FILES_FOLDER, '/') .$dest_path.$regs[1])) {
          return false;
        }
      
      $db->query("UPDATE `Filetable` SET `File_Path`='".$dest_path."' WHERE `ID` = '".$file_id."'");
      }
    }

  }

  // сообствено перемещение
  $db->query("UPDATE Message{$class_id}
                 SET Subdivision_ID={$dest_subclass[Subdivision_ID]},
                     Sub_Class_ID={$dest_subclass[Sub_Class_ID]},
                     Priority = {$dest_subclass[Next_Priority]}
               WHERE Message_ID = {$message_id}");

  // обновление приоритетов
  $db->query("UPDATE Message{$class_id}
                 SET Created=Created, LastUpdated=LastUpdated,
                     Priority = Priority-1
               WHERE Sub_Class_ID = {$message[Sub_Class_ID]}
                 AND Priority > $message[Priority]");
  
  // execute core action
  $nc_core->event->execute("updateMessage", $dest_subclass['Catalogue_ID'], $dest_subclass['Subdivision_ID'], 
                                            $dest_subclass['Sub_Class_ID'], $dest_subclass['Class_ID'], $message_id);
  // перемещение детей
  $childs_id = $db->get_col("SELECT `Message_ID` FROM `Message".$class_id."` WHERE `Parent_Message_ID` = '".$message_id."' ");
  
  if ( !empty($childs_id) ) {
    foreach ($childs_id as $child_id)
      nc_move_message($class_id, $child_id, $destination_cc_id);
  }

  return true;

}


/**
  * Копирование объекта из одного шаблона в разделе в другой.
  *
  * Пользователь должен обладать правами: изменение в разделе, где
  * находится объект, и удаление в разделе, куда переносится объект.
  *
  * @param integer ID класса объекта
  * @param integer ID объекта
  * @param integer ID шаблона в разделе, куда переносится объект
  * @return boolean
  */
function nc_copy_message($class_id, $message_id, $destination_cc_id) { 
  global $nc_core, $db;
  
  $class_id = (int)$class_id;
  $message_id = (int)$message_id;
  $destination_cc_id = (int)$destination_cc_id;

  if (!$class_id || !$message_id || !$destination_cc_id) {
    trigger_error("Wrong parameters for nc_copy_message()", E_USER_WARNING);
    return false;
  }


  // данные о месте назначения
  $dest_subclass = $db->get_row("SELECT sd.Catalogue_ID,
                                        sc.Sub_Class_ID,
                                        sc.Subdivision_ID,
                                        sc.Class_ID,
                                        IFNULL(MAX(m.Priority)+1,1) as Next_Priority
                                   FROM (Sub_Class as sc,
                                        Subdivision as sd)
                                        LEFT JOIN Message{$class_id} as m
                                          ON m.Sub_Class_ID=sc.Sub_Class_ID
                                  WHERE sc.Sub_Class_ID=$destination_cc_id
                                    AND sc.Subdivision_ID=sd.Subdivision_ID
                                  GROUP BY m.Sub_Class_ID
                                  ", ARRAY_A); 

  // переносимый объект
  $message = $db->get_row("SELECT * FROM Message{$class_id} WHERE Message_ID = $message_id", ARRAY_A);

  
  if (!$message || !$dest_subclass) {
    $what = ($message ? 'subclass' : 'object');
    trigger_error("nc_copy_message: $what doesn't exist", E_USER_WARNING);
    return false;
  }

  if ($dest_subclass['Class_ID'] != $class_id) {
    trigger_error("nc_copy_message: destination subclass belongs to different class", E_USER_WARNING);
    return false;
  }

//  if ($dest_subclass['Sub_Class_ID'] == $message['Sub_Class_ID']) { return true; } // Проверка на копирование объекта внутри одного $cc


  // права
  // Пользователь должен обладать правами: чтение в разделе, где
  // находится объект, и добавление в разделе, куда переносится объект.
  global $perm;
  $has_rights = false;
  
  $has_rights = ( $perm->isSubClass($message['Sub_Class_ID'], 48) && $perm->isSubClass($dest_subclass['Sub_Class_ID'], 48) );

  if (!$has_rights) {
    trigger_error("nc_copy_message: insufficient rights", E_USER_WARNING);
    return false;
  } // end of права

  global $AUTH_USER_ID, $HTTP_USER_AGENT;

  $message['Message_ID'] = '';
  $message['Subdivision_ID'] = $dest_subclass['Subdivision_ID'];
  $message['Sub_Class_ID'] = $dest_subclass['Sub_Class_ID'];
  $message['Priority'] = $dest_subclass['Next_Priority'];
  $message['Created'] = $message['LastUpdated'] = date("Y-m-d H:i:s");
  $message['UserAgent'] = $message['LastUserAgent'] = $HTTP_USER_AGENT;
  $message['IP'] = $message['LastIP'] = getenv("REMOTE_ADDR");
  $message['Keyword'] = '';
  $message['User_ID'] = $message['LastUser_ID'] = $AUTH_USER_ID;

  $col_names = array_keys($message);
  if ( !empty($col_names) ) {
    foreach ( $col_names as $k => $v ) $col_names[$k] = "`".$v."`";
    $col_names_string = join(", ", $col_names);
  }
	
  $col_values = array_values($message);
  foreach ( $col_values as &$value ) {
    $value=$db->escape( $value );
  }
  $col_values_string = join("', '", $col_values);
	
  $db->query("INSERT INTO Message{$class_id} (".$col_names_string.") VALUES ('".$col_values_string."')");

  $new_message_id = $db->insert_id;
  
  // execute core action
  $nc_core->event->execute("addMessage", $dest_subclass['Catalogue_ID'], $dest_subclass['Subdivision_ID'], $dest_subclass['Sub_Class_ID'], $class_id, $new_message_id);

   // копирование детей
  $childs_id = $db->get_col("SELECT `Message_ID` FROM `Message".$class_id."` WHERE `Parent_Message_ID` = '".$message_id."' ");

  if ( !empty($childs_id) ) {
    foreach ($childs_id as $child_id) {
      $new_child_id = nc_copy_message($class_id, $child_id, $destination_cc_id);
      // у дочернего объекта Parent message id остался от копируемого объекта
      $db->query("UPDATE `Message".$class_id."` SET `Parent_Message_ID` = '".$new_message_id."' WHERE `Message_ID` = '".$new_child_id."' ");

    }

  }


  // файлы
  // Поля типа "файл" в компоненте
  $file_fields = $db->get_results("SELECT `Field_ID`, `Format`, `Field_Name` 
                                    FROM `Field` 
                                    WHERE Class_ID='".$class_id."' 
                                    AND TypeOfData_ID='".NC_FIELDTYPE_FILE."'", ARRAY_A);
  
    // Если полей нет, то можно завершать работу функции
  if ( empty($file_fields) ) return $new_message_id;
 
  // prepare dirs
  global $FILES_FOLDER, $DIRCHMOD, $DOCUMENT_ROOT, $SUB_FOLDER;
  require_once($GLOBALS['INCLUDE_FOLDER'] . "s_common.inc.php");
  
  // проходим по каждому полю
  foreach ( $file_fields as $field ) {
    // если нету файл у исходного объекта - то переходим к следующему полю
    if ( !$message[$field['Field_Name']] ) continue;
    
    // определение файловой системы для записи нового файла
    $parsedFormat = nc_field_parse_format( $field['Format'], NC_FIELDTYPE_FILE );
    $fs = $parsedFormat['fs'];  
    
    //исходный файл
    $src_file_path = nc_file_path($class_id, $message_id, $field['Field_ID']); // его путь
    $value_array = explode(':', $message[$field['Field_Name']]);
    $src_file_name = $value_array[0]; // оригинальное имя
    $src_file_type = $value_array[1]; // тип
    $src_file_size = $value_array[2]; // размер
    $ext =  substr($src_file_name, strrpos($src_file_name, ".")); // расширение
    
    // в зависимости от ФС менятеся папка и имя на диске + значени в БД
    switch ( $fs ) {
      case NC_FS_SIMPLE:
        $put_file_name = $field['Field_ID']."_".$new_message_id.$ext; // имя файла: IDполя_IDобъекта.расширение
        $FilePath = ''; // в папку netcat_files
        $fieldValue = $src_file_name.":".$src_file_type.":".$src_file_size; // значение в базу
        break;
      case NC_FS_ORIGINAL:
        $put_file_name = nc_transliterate($src_file_name);
        
        $k = 0;
		    // файл с таким именем может существовать - нужно добавить индекс
		    if ( file_exists($FILES_FOLDER.$put_file_name) )  {
		      while ( file_exists($FILES_FOLDER.substr($put_file_name,0, strrpos($put_file_name, "."))."_".$k.$ext) )  $k++; 
		      
		      $put_file_name = substr($put_file_name,0, strrpos($put_file_name, "."))."_".$k.$ext;
		    }
		    $FilePath = ''; // в папку netcat_files
		    $fieldValue = $src_file_name.":".$src_file_type.":".$src_file_size.":".$put_file_name; // значение в базу
		    break;
      case NC_FS_PROTECTED:
        #check and create dirs
        $dirs = array($FILES_FOLDER . $dest_subclass['Subdivision_ID'],
                $FILES_FOLDER . $dest_subclass['Subdivision_ID'] . '/' . $dest_subclass['Sub_Class_ID']);
        foreach ($dirs as $dir) {
          if (!file_exists($dir) && !mkdir($dir, $DIRCHMOD)) { return false; } // can't create dir
        }
        // директория
        $FilePath = "/$dest_subclass[Subdivision_ID]/$dest_subclass[Sub_Class_ID]/";
        // имя файла
        $put_file_name = md5($src_file_name.date("H:i:s d.m.Y").uniqid("NetCat"));
        // в таблицу Filetable
        $db->query("INSERT INTO Filetable (ID, Real_Name, Virt_Name, File_Path, File_Type, File_Size, Message_ID, Field_ID) 
          VALUES ('', '".$src_file_name."', '".$put_file_name."', '".$FilePath."', '".$src_file_type."', 
          '".$src_file_size."', '".$new_message_id."', '".$field['Field_ID']."')");
        $fieldValue = $src_file_name.":".$src_file_type.":".$src_file_size; // значение в базу
        break;
        
    }
    
    // копирование файла
    copy($DOCUMENT_ROOT.$src_file_path, $FILES_FOLDER.$FilePath.$put_file_name);
    // правка в БД
    $db->query("UPDATE `Message".$class_id."` SET `".$field['Field_Name']."` = '".$fieldValue."' WHERE `Message_ID`='".$new_message_id."'");
  }
  

  return $new_message_id;
}


/**
  * Перемещение шаблона-в-разделе в другой раздел
  */
function nc_move_subclass($subclass_id, $dest_sub_id) {
  global $db, $perm, $nc_core;
  
  $subclass_id = intval($subclass_id);
  $dest_sub_id = intval($dest_sub_id);
  
  $subclass = $db->get_row("SELECT Class_ID, Catalogue_ID, Subdivision_ID, Sub_Class_ID, Priority FROM Sub_Class WHERE Sub_Class_ID = $subclass_id", ARRAY_A);
  $subdivision = $db->get_row("SELECT Catalogue_ID, Subdivision_ID FROM Subdivision WHERE Subdivision_ID = $dest_sub_id", ARRAY_A);

  if (!$subclass || !$subdivision) {
    trigger_error("nc_move_subclass: wrong parameters (can't get data)", E_USER_WARNING);
    return false;
  }

  $has_rights = false;
  
  $has_rights = $perm->isSubdivisionAdmin($dest_sub_id);

  if (!$has_rights) {
    trigger_error("nc_move_subclass: insufficient rights", E_USER_WARNING);
    return false;
  }

  if ($subclass['Subdivision_ID']==$dest_sub_id) { return false; }

  // move subdivision
  $next_priority = $db->get_var("SELECT IFNULL(MAX(Priority)+1,1)
                                   FROM Sub_Class
                                  WHERE Subdivision_ID = $dest_sub_id");

  $db->query("UPDATE Sub_Class
                 SET Subdivision_ID=$dest_sub_id, Priority=$next_priority
               WHERE Sub_Class_ID=$subclass_id");

  // close gap priority at old subdivision
  $db->query("UPDATE Sub_Class
                 SET Priority=Priority-1
               WHERE Subdivision_ID = $subclass[Subdivision_ID]
                 AND Priority > $subclass[Priority]");

  // move 'messages'
  $db->query("UPDATE Message{$subclass[Class_ID]}
                 SET Subdivision_ID = $dest_sub_id
               WHERE Sub_Class_ID = $subclass_id");

  // move files
  $has_files = $db->get_var("SELECT COUNT(*)
                               FROM Filetable
                              WHERE File_Path = '/$subclass[Subdivision_ID]/$subclass[Sub_Class_ID]/'");
  if ($has_files) {
    // prepare dirs
    global $FILES_FOLDER, $DIRCHMOD;
    @mkdir($FILES_FOLDER."$dest_sub_id", $DIRCHMOD);

    // move dir
    $old_path = $FILES_FOLDER . "$subclass[Subdivision_ID]/$subclass[Sub_Class_ID]";
    $new_path = $FILES_FOLDER . "$dest_sub_id/$subclass_id";

    if (file_exists($old_path)) {
      rename($old_path, $new_path);
    }

    $db->query("UPDATE Filetable
                   SET File_Path='/$dest_sub_id/$subclass_id/'
                 WHERE File_Path='/$subclass[Subdivision_ID]/$subclass[Sub_Class_ID]/'");
  }
  
  // execute core action
  $nc_core->event->execute("updateSubClass", $subdivision['Catalogue_ID'], $subdivision['Subdivision_ID'], $subclass_id);

  return true;
}

/**
  * Отображение данных(функций, переменных) для вставки в textarea формы
  * @param string $type тип формы (template || class)
  * @param string $window окно для вставки данных, например window или opener
  * @param string $form идентификатор формы
  * @param string $textarea идентификатор textarea
  */
function nc_form_data_insert($type, $window, $form, $textarea) {
  global $db;
//  global $class_id, $system_class_id;

  $class_id = intval($_GET['classid']);
  $system_class_id = intval($_GET['systemclassid']);

  
  
  switch ($type) {
    case "class":
      if ( !$class_id &&! $system_class_id) return;
      if ($textarea == 'PageBody' || $textarea == 'ListBody') {
         // все поля компонента
         $class_fields = $db->get_results("SELECT `Field_Name`, `Description`, `TypeOfData_ID` as `type`
                                            FROM `Field` 
                                            WHERE ".( $class_id ? 
                                               "`Class_ID` = '".$class_id."'":
                                               "`System_Table_ID` = '".$system_class_id."'" )."
                                            ORDER BY `Priority`", ARRAY_A);
        
        
        if ( !empty($class_fields) ) {
            // Поля компонента
            echo "<table class='InsertDataTable'>\n";
            echo "<tr><td colspan='2'><b>".NETCAT_HINT_COMPONENT_FIELD."</b></td></tr>";
            foreach ($class_fields as $class_field) {
              echo "<tr><td>
                      <a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_".$class_field['Field_Name']."</a>
                    </td><td>".$class_field['Description']."</td></tr>\n";

              if ( $class_field['type'] == NC_FIELDTYPE_FILE ) {
                 echo "<tr><td>
                      <a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_".$class_field['Field_Name']."_size</a>
                    </td><td>".NETCAT_HINT_COMPONENT_SIZE." ".$class_field['Description']."</td></tr>\n";
                 echo "<tr><td>
                      <a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_".$class_field['Field_Name']."_type</a>
                    </td><td>".NETCAT_HINT_COMPONENT_TYPE." ".$class_field['Description']."</td></tr>\n";
              }

              if ( $class_field['type'] == NC_FIELDTYPE_SELECT || $class_field['type'] == NC_FIELDTYPE_MULTISELECT ) {
                 echo "<tr><td>
                      <a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_".$class_field['Field_Name']."_id</a>
                    </td><td>".NETCAT_HINT_COMPONENT_ID." ".$class_field['Description']."</td></tr>\n";
              }

              if ( $class_field['type'] == NC_FIELDTYPE_DATETIME ) {
                 echo "<tr><td>
                      <a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_".$class_field['Field_Name']."_day</a>
                    </td><td>".NETCAT_HINT_COMPONENT_DAY." ".$class_field['Description']."</td></tr>\n";
                 echo "<tr><td>
                      <a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_".$class_field['Field_Name']."_month</a>
                    </td><td>".NETCAT_HINT_COMPONENT_MONTH." ".$class_field['Description']."</td></tr>\n";
                 echo "<tr><td>
                      <a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_".$class_field['Field_Name']."_year</a>
                    </td><td>".NETCAT_HINT_COMPONENT_YEAR." ".$class_field['Description']."</td></tr>\n";
                 echo "<tr><td>
                      <a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_".$class_field['Field_Name']."_hours</a>
                    </td><td>".NETCAT_HINT_COMPONENT_HOUR." ".$class_field['Description']."</td></tr>\n";
                 echo "<tr><td>
                      <a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_".$class_field['Field_Name']."_minutes</a>
                    </td><td>".NETCAT_HINT_COMPONENT_MINUTE." ".$class_field['Description']."</td></tr>\n";
                 echo "<tr><td>
                      <a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_".$class_field['Field_Name']."_seconds</a>
                    </td><td>".NETCAT_HINT_COMPONENT_SECONDS." ".$class_field['Description']."</td></tr>\n";
              }
            }
            echo "</table>\n<br><br>\n";
          }
      


        // Переменные, содержащие свойства текущего объекта
        echo "<table class='InsertDataTable'>\n";
        echo "<tr><td colspan='2'><b>".NETCAT_HINT_OBJECT_PARAMS."</b></td></tr>";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_Created</a></td><td>".NETCAT_HINT_CREATED_DESC."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_LastUpdated</a></td><td>".NETCAT_HINT_LASTUPDATED_DESC."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_RowID</a></td><td>".NETCAT_HINT_MESSAGE_ID."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_UserID</a></td><td>".NETCAT_HINT_USER_ID."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_Checked</a></td><td>".NETCAT_HINT_CHECKED."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_IP</a></td><td>".NETCAT_HINT_IP."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_UserAgent</a></td><td>".NETCAT_HINT_USER_AGENT."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_LastUserID</a></td><td>".NETCAT_HINT_LAST_USER_ID."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_LastIP</a></td><td>".NETCAT_HINT_LAST_USER_IP."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_LastUserAgent</a></td><td>".NETCAT_HINT_LAST_USER_AGENT."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_AdminButtons</a></td><td>".NETCAT_HINT_ADMIN_BUTTONS."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_AdminCommon</a></td><td>".NETCAT_HINT_ADMIN_COMMONS."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$fullLink</a></td><td>".NETCAT_HINT_FULL_LINK."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$fullDateLink</a></td><td>".NETCAT_HINT_FULL_DATE_LINK."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$editLink</a></td><td>".NETCAT_HINT_EDIT_LINK."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$deleteLink</a></td><td>".NETCAT_HINT_DELETE_LINK."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$dropLink</a></td><td>".NETCAT_HINT_DROP_LINK."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$checkedLink</a></td><td>".NETCAT_HINT_CHECKED_LINK."</td></tr>\n";
        echo "</table>\n";

        // Переменные, доступные в списке объектов шаблона
        echo "<table class='InsertDataTable'>\n";
        echo "<tr><td colspan='2'><b>".NETCAT_HINT_VARS_IN_LIST_SCOPE."</b></td></tr>";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$prevLink</a></td><td>".NETCAT_HINT_PREV_LINK."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$nextLink</a></td><td>".NETCAT_HINT_NEXT_LINK."</td></tr>\n";
        if ($textarea == 'PageBody') echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_RowNum</a></td><td>".NETCAT_HINT_ROW_NUM."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$recNum</a></td><td>".NETCAT_HINT_REC_NUM."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$totRows</a></td><td>".NETCAT_HINT_TOT_ROWS."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$begRow</a></td><td>".NETCAT_HINT_BEG_ROWS."</td></tr>\n";
        echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$endRow</a></td><td>".NETCAT_HINT_END_ROWS."</td></tr>\n";
        echo "</table>\n";
      }

      // Переменные, доступные во всех полях шаблона
      echo "<table class='InsertDataTable'>\n";
      echo "<tr><td colspan='2'><b>".NETCAT_HINT_VARS_IN_COMPONENT_SCOPE."</b></td></tr>";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$admin_mode</a></td><td>".NETCAT_HINT_ADMIN_MODE."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$subHost</a></td><td>".NETCAT_HINT_SUB_HOST."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$subLink</a></td><td>".NETCAT_HINT_SUB_LINK."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$ccLink</a></td><td>".NETCAT_HINT_CC_LINK."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$catalogue</a></td><td>".NETCAT_HINT_CATALOGUE_ID."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$sub</a></td><td>".NETCAT_HINT_SUB_ID."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$cc</a></td><td>".NETCAT_HINT_CC_ID."</td></tr>\n";
      echo "</table>\n";

      // Хэш-массивы
      echo "<table class='InsertDataTable'>\n";
      echo "<tr><td colspan='2'><b>".NETCAT_HINT_ARRAY."</b></td></tr>";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$current_catalogue[]</a></td><td>".NETCAT_HINT_CURRENT_CATALOGUE."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$current_sub[]</a></td><td>".NETCAT_HINT_CURRENT_SUB."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$current_сс[]</a></td><td>".NETCAT_HINT_CURRENT_CC."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$current_user[]</a></td><td>".NETCAT_HINT_CURRENT_USER."</td></tr>\n";
      echo "</table>\n";
      
      // Функции
      echo "<table class='InsertDataTable'>\n";
      echo "<tr><td colspan='2'><b>".NETCAT_HINT_FUNCTION."</b></td></tr>";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">is_even(int \$param)</a></td><td>".NETCAT_HINT_IS_EVEN."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">opt(\$flag, \$string)</a></td><td>".NETCAT_HINT_OPT."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">opt_case(\$flag, \$string1, \$string2)</a></td><td>".NETCAT_HINT_OPT_CAES."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">listQuery(char \$sql_query, char \$output_template = NULL, char \$divider = NULL)</a></td><td>".NETCAT_HINT_LIST_QUERY."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">nc_list_select(\$classificator_name, \$field_name = NULL, \$current_value = NULL, \$sort_type = NULL, \$sort_direction = NULL, \$template_prefix = NULL, \$template_object = NULL, \$template_suffix = NULL, \$template_any = NULL)</a></td><td>".NETCAT_HINT_NC_LIST_SELECT."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">nc_message_link(int \$message_id, int \$class_id)</a></td><td>".NETCAT_HINT_NC_MESSAGE_LINK."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">nc_file_path( mixed \$class_id, int \$message_id, mixed \$field_name_or_id, [string \$file_name_prefix=\"\"] )</a></td><td>".NETCAT_HINT_NC_FILE_PATH."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">browse_messages(\$cc_env, \$range)</a></td><td>".NETCAT_HINT_BROWSE_MESSAGE."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">nc_objects_list(int \$sub, int \$cc, char \$params, bool \$show_in_admin_mode = FALSE)</a></td><td>".NETCAT_HINT_NC_OBJECTS_LIST."</td></tr>\n";
      echo "<tr><td colspan='2'><b>".NETCAT_HINT_RTFM."</b></td></tr>";
      echo "</table>\n";
    break;
    case "template":
      // Переменные, доступные во всех полях шаблона
      echo "<table class='InsertDataTable'>\n";
      echo "<tr><td colspan='2'><b>".NETCAT_HINT_VARS_IN_COMPONENT_SCOPE."</b></td></tr>";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$catalogue</a></td><td>".NETCAT_HINT_CATALOGUE_ID."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$sub</a></td><td>".NETCAT_HINT_SUB_ID."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$cc</a></td><td>".NETCAT_HINT_CC_ID."</td></tr>\n";
      echo "</table>\n";

      // Хэш-массивы
      echo "<table class='InsertDataTable'>\n";
      echo "<tr><td colspan='2'><b>".NETCAT_HINT_ARRAY."</b></td></tr>";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$current_catalogue[]</a></td><td>".NETCAT_HINT_CURRENT_CATALOGUE."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$current_sub[]</a></td><td>".NETCAT_HINT_CURRENT_SUB."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$current_сс[]</a></td><td>".NETCAT_HINT_CURRENT_CC."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$current_user[]</a></td><td>".NETCAT_HINT_CURRENT_USER."</td></tr>\n";
      echo "</table>\n";

      // Функции
      echo "<table class='InsertDataTable'>\n";
      echo "<tr><td colspan='2'><b>".NETCAT_HINT_FUNCTION."</b></td></tr>";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">is_even(int \$param)</a></td><td>".NETCAT_HINT_IS_EVEN."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">opt(\$flag, \$string)</a></td><td>".NETCAT_HINT_OPT."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">opt_case(\$flag, \$string1, \$string2)</a></td><td>".NETCAT_HINT_OPT_CAES."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">listQuery(char \$sql_query, char \$output_template = NULL, char \$divider = NULL)</a></td><td>".NETCAT_HINT_LIST_QUERY."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">nc_message_link(int \$message_id, int \$class_id)</a></td><td>".NETCAT_HINT_NC_MESSAGE_LINK."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">nc_file_path( mixed \$class_id, int \$message_id, mixed \$field_name_or_id, [string \$file_name_prefix=\"\"] )</a></td><td>".NETCAT_HINT_NC_FILE_PATH."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">nc_objects_list(int \$sub, int \$cc, char \$params, bool \$show_in_admin_mode = FALSE)</a></td><td>".NETCAT_HINT_NC_OBJECTS_LIST."</td></tr>\n";
      echo "<tr><td colspan='2'><b>".NETCAT_HINT_RTFM."</b></td></tr>";
      echo "</table>\n";
    break;
    
    case "classinput":
      if ($class_id) {
        $class_fields = $db->get_results("SELECT Field_Name, Description FROM Field WHERE Class_ID = '".$class_id."' ORDER BY Priority", ARRAY_A);
      } else {
        if ($system_class_id) $class_fields = $db->get_results("SELECT Field_Name, Description FROM Field WHERE System_Table_ID = '".$system_class_id."' ORDER BY Priority", ARRAY_A);  
      }
      if ($class_fields) {
        // Поля шаблона
        echo "<table class='InsertDataTable'>\n";
        echo "<tr><td colspan='2'><b>".NETCAT_HINT_COMPONENT_FIELD."</b></td></tr>";
        foreach ($class_fields as $class_field) {
          echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">".$class_field['Field_Name']."</a></td><td>".$class_field['Description']."</td></tr>\n";
        }
        echo "</table>\n<br><br>\n";
      }
      echo "<table class='InsertDataTable'>\n";
      echo "<tr><td colspan='2'><b>".NETCAT_HINT_OBJECT_PARAMS."</b></td></tr>";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_Created</a></td><td>".NETCAT_HINT_CREATED_DESC."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_LastUpdated</a></td><td>".NETCAT_HINT_LASTUPDATED_DESC."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_RowID</a></td><td>".NETCAT_HINT_MESSAGE_ID."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_UserID</a></td><td>".NETCAT_HINT_USER_ID."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_Checked</a></td><td>".NETCAT_HINT_CHECKED."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_IP</a></td><td>".NETCAT_HINT_IP."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_UserAgent</a></td><td>".NETCAT_HINT_USER_AGENT."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_LastUserID</a></td><td>".NETCAT_HINT_LAST_USER_ID."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_LastIP</a></td><td>".NETCAT_HINT_LAST_USER_IP."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_LastUserAgent</a></td><td>".NETCAT_HINT_LAST_USER_AGENT."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_AdminButtons</a></td><td>".NETCAT_HINT_ADMIN_BUTTONS."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$f_AdminCommon</a></td><td>".NETCAT_HINT_ADMIN_COMMONS."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$fullLink</a></td><td>".NETCAT_HINT_FULL_LINK."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$fullDateLink</a></td><td>".NETCAT_HINT_FULL_DATE_LINK."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$editLink</a></td><td>".NETCAT_HINT_EDIT_LINK."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$deleteLink</a></td><td>".NETCAT_HINT_DELETE_LINK."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$dropLink</a></td><td>".NETCAT_HINT_DROP_LINK."</td></tr>\n";
      echo "<tr><td><a href='#' title='' onclick=\"insert_bbcode('".$window."', '".$form."', '".$textarea."', this.innerHTML, '', false);\">\$checkedLink</a></td><td>".NETCAT_HINT_CHECKED_LINK."</td></tr>\n";
      echo "</table>\n";
    break;
  }
}


/**
  * Вспомогательная рекурсивная функция для получения всех идентификаторов 
  * подразделов раздела с идентификатором $subdivision_id
  * @param int $subdivision_id идентификатор раздела
  * @return array идентификаторы подразделов
  */
function nc_get_subdivision_children($subdivision_id) {
  global $db;

  $subdivision_id = intval($subdivision_id);
  $parent_subs = $db->get_col("SELECT Subdivision_ID FROM Subdivision WHERE Parent_Sub_ID = $subdivision_id");

  if (empty($parent_subs)) return array();
  foreach ($parent_subs as $parent_sub_id) {
    $parent_subs_array[] = $parent_sub_id;
	$parent_subs_array = array_merge($parent_subs_array, nc_get_subdivision_children($parent_sub_id));
  }
  
  return $parent_subs_array;
}


function nc_admin_input($disc, $name, $value = '', $size = 0, $style = '') {
 if ( is_array($value) ) $value = $value[$name];
 return "<div style='margin:10px 0; _padding:0;'>".$disc.":<br/>
       <input type='text' name='".$name."' ".($size ? "size='".$size."'" : "")." ".( $style ? "style='".$style."'" : "")." value='".htmlentities($value, ENT_QUOTES, MAIN_ENCODING)."'/></div>\n";
}

function nc_admin_input_in_text ($disc, $name, $value = '', $size = 0, $style = '') {
  if ( is_array($value) ) $value = $value[$name];
  return str_replace('%input',"<input type='text' name='".$name."' ".($size ? "size='".$size."'" : "")." ".( $style ? "style='".$style."'" : "")." value='".htmlentities($value, ENT_QUOTES, MAIN_ENCODING)."'/>\n" , $disc)."\n";
}


function nc_admin_img ( $name, $alt, $width = 0, $height = 0, $align = '' ) {
  global $ADMIN_TEMPLATE;
  return  "<img src='".$ADMIN_TEMPLATE."img/".$name."' ".( $width ? "width='".$width."'" : "" )." ".( $height ? "height='".$height."'" : "" )."  ".( $align ? "align='".$align."'" : "" )." alt='".$alt."' title='".$alt."' />";
}


function nc_admin_textarea ( $disc, $name, $value, $is_resizeble = 0, $fck = 0, $style = '' ) {
   if ( is_array($value) ) $value = $value[$name];
   $nc_core = nc_Core::get_object();
   if ( !$style ) $style = "margin-top: 5px; width:100%; height:6em; line-height:1em";
   $ret = "<div style='margin:10px 0; _padding:0;'>\n".$disc.":";
   if ( $fck ) {
     $ret .= "<button type='button' onclick=\"window.open('".$nc_core->SUB_FOLDER.$nc_core->HTTP_ROOT_PATH."editors/FCKeditor/neditor.php?form=main&control=".$name."', 'Editor', 'width=750,height=605,resizable=yes,scrollbars=no,toolbar=no,location=no,status=no,menubar=no');\">".TOOLS_HTML_INFO."</button><br>";
   }
   if ( $is_resizeble ) {
     $ret .= "<div class='resize_block'><a class='textarea_shrink' href='#".$name."' > &#x25BC; </a>
                                     <a class='textarea_grow' href='#".$name."'> &#x25B2; </a></div>";
   }

   $ret .= "
        <textarea name='".$name."' id='".$name."' style='".$style."'>".htmlentities($value, ENT_COMPAT, MAIN_ENCODING)."</textarea>".
         "</div>\n";

   return $ret;
 }


function nc_admin_checkbox ( $disc, $name, $value, $style = '' ) {
    if ( is_array($value) ) $value = $value[$name];
    return " <div style='margin:5px 0; _padding:0;'>\n
        <input type='checkbox' name='".$name."' id='".$name."' ".($style ? $style : "")." value='1'".($value ? " checked" : "")."/>
        <label for='".$name."'>".$disc."</label>\n</div>\n";
 }


function nc_admin_classificators ($disc, $name, $value = '', $style = '') {
  $nc_core = nc_Core::get_object();
  static $cf = array();

  if ( empty($cf) ) {
    $res = $nc_core->db->get_results("SELECT `Classificator_Name`, `Table_Name`
                                      FROM `Classificator`
                                      WHERE `System` = 0 ORDER BY `Classificator_ID` ", ARRAY_A);
    if ( !empty($res) ) foreach ( $res as $v ) {
      $cf[$v['Table_Name']] = $v['Classificator_Name'];
    }
  }

  $res = $disc.":<br/>";
  $res .= "<select name='".$name."' ".( $style ? "style='".$style."'" : "").">";
  foreach ( $cf as $k => $v ) {
    $res .= "<option value='".$k."' ".($value == $k ? " selected='selected' " : "").">".$v."</option>";
  }
  $res .= "</select>";

  return $res;
}


function nc_admin_select_static( $disc, $name, $value = array() ) {
  $nc_core = nc_Core::get_object();
  $ret = $disc.":<br/>";
  $ret .= "<div id='select_static'>
            <div id='select_static_head'>
              <div class='key'>".NETCAT_CUSTOM_KEY."</div>
              <div class='value'>".NETCAT_CUSTOM_VALUE."</div>
              <div style='clear:both;'></div>
            </div>
          </div>";
  $ret .="<div onclick='nc_s.add()' class='select_static_add'>
            <img src='".$nc_core->ADMIN_TEMPLATE."img/i_obj_add.gif' alt='' />".CONTROL_CONTENT_SUBDIVISION_FUNCS_ADD."
         </div>";

  $ret .= "<script> nc_s = new nc_selectstatic();\r\n";
  if ( !empty ($value) )foreach ( $value as $k => $v  ) {
    $ret .= "\tnc_s.add('".$k."', '".$v."'); \r\n";
  }
  $ret .= "</script>";

  return $ret;
}

function nc_admin_select_subdivision ( $disc, $name, $value ) {
  global $db;
  static $subs;

  if ( is_array($value) ) $value = $value[$name];
  if ( !$subs ) {
    $subs = $db->get_results("SELECT s.`Subdivision_ID` as `value`,
      CONCAT(s.`Subdivision_ID`, '. ', s.`Subdivision_Name`) as  `description`,
      c.`Catalogue_Name` as `optgroup`,
      s.`Parent_Sub_ID` as `parent`
      FROM `Catalogue` AS `c`, `Subdivision` AS `s`
      WHERE s.`Catalogue_ID` = c.`Catalogue_ID`
      ORDER BY c.`Priority`, s.`Priority` ", ARRAY_A);
  }

  $res  = "<div style='margin:5px 0; _padding:0;'>".$disc." ";
  $res .= "<select name='".$name."'>".nc_select_options($subs, $value)."</select></div>";
  return $res;
}

function nc_admin_select_component ( $disc, $name, $value = '') {
  global $db;
  static $classes;
  
  if ( is_array($value) ) $value = $value[$name];
  if ( !$classes ) {
   $classes  = $db->get_results("SELECT `Class_ID` as value,
      CONCAT(`Class_ID`, '. ', `Class_Name`) as description,
      `Class_Group` as optgroup
      FROM `Class`
      WHERE `ClassTemplate` = 0
      ORDER BY `Class_Group`, `Class_ID`", ARRAY_A);
  }

  $res  = "<div style='margin:5px 0; _padding:0;'>".$disc." ";
  $res .= "<select name='".$name."'>".nc_select_options($classes, $value)."</select></div>";
  return $res;
}

function nc_admin_select_field ( $table, $disc, $name, $value = '' ) {
  $c = new nc_Component(0, 3);
  $fields = $c->get_fields();
  if ( is_array($value) ) $value = $value[$name];
  
  $res  = "<div style='margin:5px 0; _padding:0;'>".$disc." ";
  if ( !empty($fields) ) {
    $res .= "<select name='".$name."'>\n";
    foreach ( $fields as $v ) {
      $res .="\t<option value='".$v['name']."' ".( $value == $v['name'] ? "selected='selected'" : "").">".$v['description']."</option>\n";
    }
    $res .= "</select>\n";
  }
  $res .= "</div>";
  return $res;
}

function nc_admin_select_usergroup ( $disc, $name, $value ='' ) {
  global $db;
  $groups = $db->get_results("SELECT * FROM `PermissionGroup`", ARRAY_A);

  $res  = "<div style='margin:5px 0; _padding:0;'>".$disc." ";
  if ( !empty($groups) ) {
    $res .= "<select name='".$name."'>\n";
    foreach ( $groups as $v ) {
      $res .="\t<option value='".$v['PermissionGroup_ID']."' ".( $value == $v['PermissionGroup_ID'] ? "selected='selected'" : "").">".$v['PermissionGroup_Name']."</option>\n";
    }
    $res .= "</select>\n";
  }
  $res .= "</div>";
  return $res;
}
/**
 * Функция возвращает js-код для ресайза текстовых полей
 * @return string js-код
 */
function nc_admin_js_resize () {
   return " <script type='text/javascript'>
    (function (){
      var links = document.getElementsByTagName(\"A\");
      var onclicker_grow   = function (event) { ShrinkArea(event, 50);  }
      var onclicker_shrink = function (event) { ShrinkArea(event,-50);  }
      for (var i = 0 ; i < links.length ; i++){
          if (links[i].className.search('textarea_shrink') != -1 ) {
            bindEvent(links[i], 'click', onclicker_grow );
          }
          if (links[i].className.search('textarea_grow') != -1 ) {
            bindEvent(links[i], 'click', onclicker_shrink);
          }
      }
    }) ();
  </script>";
 }

/**
 * Функция выводит список пользовательских настроек
 *
 * @param int номер компонента
 * @param int номер макета дизайн
 * @param array массив с настройками
 * @return int
 */
function nc_customsettings_show ( $ClassID =0, $TemplateID = 0, $custom_settings = array() ) {
  global $UI_CONFIG;

  if ( !$ClassID && !$TemplateID ) return false;
  
  if ( $ClassID ) {
    $ac = "urlDispatcher.load('dataclass.custom.new(".$ClassID.")')";
    $ac1 = "urlDispatcher.load('dataclass.custom.manual(".$ClassID.")')";
  }
  else {
    $ac = "urlDispatcher.load('template.custom.new(".$TemplateID.")')";
    $ac1 = "urlDispatcher.load('template.custom.manual(".$TemplateID.")')";
  }

  $UI_CONFIG->actionButtons[] = array("id" => "addcs",
                                        "caption" => CONTROL_FIELD_LIST_ADD,
                                        "action" => $ac );
  $UI_CONFIG->actionButtons[] = array("id" => "del",
                                        "caption" => NETCAT_CUSTOM_ONCE_MANUAL_EDIT,
                                        "action" => $ac1);

  if ( empty($custom_settings) ) {
    nc_print_status(NETCAT_CUSTOM_NONE_SETTINGS, 'info');
    return false;
  }

  $a2f = new nc_a2f($custom_settings, '');

  $header = " <table style='border:none; background: #ccc; width: 100%;' cellpadding='5' cellspacing='1'>
            <tr>
              <td bgcolor='eeeeee' width='30%'><font size='-2'><b>".NETCAT_CUSTOM_ONCE_FIELD_NAME."</b></td>
              <td bgcolor='eeeeee' width='45%'><font size='-2'><b>".NETCAT_CUSTOM_ONCE_FIELD_DESC."</b></td>
              <td bgcolor='eeeeee'  width='20%'><font size='-2'><b>".NETCAT_CUSTOM_TYPE."</b></font></td>
              <td bgcolor=eeeeee align=center width='5%'>
                ".nc_admin_img('i_delete.gif', NETCAT_CUSTOM_ONCE_DROP)."
              </td>
            </tr>";

  echo "<form action='index.php' method='post' >";
  echo "<input type='hidden' name='phase' value='".( $ClassID ? "241" : "81")."' />";
  echo "<input type='hidden' name='ClassID' value='".$ClassID."' />";
  echo "<input type='hidden' name='TemplateID' value='".$TemplateID."' />";

  echo $a2f->settings_render($header, '<tr style="background-color: #FFF;"><td><a href="index.php?phase='.($ClassID ? 25 : 9).'&amp;'.( $ClassID ? "ClassID=".$ClassID : "TemplateID=".$TemplateID).'&amp;param=%NAME">%NAME</a></td><td>%CAPTION</td><td>%TYPENAME</td><td align="center"><input type="checkbox" name="kill[]" value="%NAME" /></td></tr>', '</table>');
  $custom_settings = $a2f->eval_value($custom_settings);

  $UI_CONFIG->actionButtons[] = array("id" => "del",
                                        "caption" => NETCAT_CUSTOM_ONCE_DROP_SELECTED,
                                        "action" => "mainView.submitIframeForm()",
                                        "align" => "left");

  

  echo "</form>";
  return false;
 
}


/**
 * Функция удаляет параметры в пользовательских настроек
 * имена удаляемые параметров передаются в массиве $_POST['kill']
 * @param int номер компонента
 * @param int номер макета дизайн
 * @param array массив с настройками
 * @return string строка с настройками
 */
function nc_customsettings_drop ( $ClassID = 0, $TemplateID = 0, $custom_settings = '' ) {
  $nc_core = nc_Core::get_object();

  if ( !$custom_settings || empty($_POST['kill']) ) return $custom_settings;

  $ClassID = intval($nc_core->input->fetch_get_post('ClassID'));
  $TemplateID = intval($nc_core->input->fetch_get_post('TemplateID'));

  $a2f = new nc_a2f($custom_settings, '');
  $custom_settings = $a2f->eval_value($custom_settings);

  foreach ( $custom_settings as $k => $v ) {
    if (in_array($k, $_POST['kill']) ) unset($custom_settings[$k]);
  }

  $s = '';
  if ( !empty($custom_settings) ) {
    $s = '$settings_array = '.nc_a2f::array_to_string($custom_settings).';';
  }

  if ( $ClassID ) {
    $nc_core->component->update($ClassID, array('CustomSettingsTemplate' => $s));
  }
  else {
    $nc_core->template->update($TemplateID, array('CustomSettings' => $s));
  }

  return $s;
}



/**
 * Функция выводит форму добавления/редактирования параметра пользовательских настроек
 *
 * @param int номер компонента
 * @param int номер макета дизайна
 * @param string имя параметра
 * @return int
 */
function nc_customsettings_show_once ( $ClassID = 0, $TemplateID = 0, $param = '' ) {
  $nc_core = nc_Core::get_object();
  global $UI_CONFIG;

  $types = array('string', 'textarea', 'int', 'float', 'datetime', 'select', 'rel', 'file', 'divider');
  $subtypes = array();
  $extend_params = array();
  foreach ( $types as $v ) {
    $classname = "nc_a2f_field_".$v;
    if ( !class_exists($classname) ) die('Incorrect type: '.$v.'. Class '.$classname.' not found.');
    $f = new $classname ();
    $extend_params[$v] = $f->get_extend_parametrs();
    $subtypes[$v] = $f->get_subtypes();
    $has_dafault[$v] = $f->has_default();
    foreach ( $subtypes[$v] as $k => $val ) {
      $classname = "nc_a2f_field_".$v."_".$val;
      if ( !class_exists($classname) ) die('Incorrect type: '.$val.'. Class '.$classname.' not found.');
      $f = new $classname ();
      $extend_params[$v."_".$val] = $f->get_extend_parametrs();
      $subtypes[$v][$k] = array( $val => constant('NETCAT_CUSTOM_TYPENAME_'.strtoupper($v).'_'.strtoupper($val) ) );
    }
  }
  //dump ( $extend_params );
  //print nc_array_json($subtypes);

  if ( $ClassID ) {
    $custom_settings = $nc_core->component->get_by_id( $ClassID , 'CustomSettingsTemplate');
  }
  else {
    $custom_settings = $nc_core->template->get_by_id( $TemplateID , 'CustomSettings');
  }

  if ( $custom_settings ) {
    $a2f = new nc_a2f($custom_settings, '');
    $custom_settings = $a2f->eval_value($custom_settings);
    $settings =  $custom_settings[$param];
  }

  if ( ($pst = $nc_core->input->fetch_get_post('type')) )  $settings['type'] = $pst;
  if ( ($pst = $nc_core->input->fetch_get_post('subtype')) )  $settings['subtype'] = $pst;
  if ( !$settings['type'] ) $settings['type'] = 'string';
  $classname = 'nc_a2f_field_'.$settings['type'];
  $cs = new $classname($settings);

  echo "<script type='text/javascript'> 
          nc_cs = new nc_customsettings('".$settings['type']."', '".( $settings['subtype'] ? $settings['subtype'] : '')."', ".nc_array_json($subtypes).", ".nc_array_json($has_dafault).");
       </script>";
  echo "<form action='index.php' method='post' >";
  echo "<input type='hidden' name='phase' value='".( $ClassID ? "251" : "91")."' />";
  echo "<input type='hidden' name='ClassID' value='".$ClassID."' />";
  echo "<input type='hidden' name='TemplateID' value='".$TemplateID."' />";
  echo "<input type='hidden' name='param' value='".$param."' />";
  echo "<input type='submit' class='hidden'>";
  echo "<fieldset style='margin-bottom: 15px;'><legend><b>".NETCAT_CUSTOM_ONCE_MAIN_SETTINGS."</b></legend><div style='width: 50%;'>";
  echo nc_admin_input ( NETCAT_CUSTOM_ONCE_FIELD_NAME, 'name', ( ($pst = $nc_core->input->fetch_get_post('name')) ? $pst : $param), 0, 'width:100%;  margin-bottom: 10px;');
  echo nc_admin_input ( NETCAT_CUSTOM_ONCE_FIELD_DESC, 'caption', ( ($pst = $nc_core->input->fetch_get_post('caption')) ? $pst : $settings['caption']), 0, 'width:100%;  margin-bottom: 10px;');
  echo "<div id='def' style='display: ".($cs->has_default() ? "block" : "none")."'>".nc_admin_input ( NETCAT_CUSTOM_ONCE_DEFAULT, 'default_value', ( ($pst = $nc_core->input->fetch_get_post('default_value')) ? $pst : $settings['default_value']), 0, 'width:100%;  margin-bottom: 10px;')."</div>";
 
  echo NETCAT_CUSTOM_TYPE.": <br/>
        <select id='type' style='width: 100%;  margin-bottom: 10px;' name='type' onchange='nc_cs.changetype()' >";
  foreach ( $types as $v ) {
    echo "<option value='".$v."' ".( $v == $settings['type'] ? "selected='selected' " : "").">".constant('NETCAT_CUSTOM_TYPENAME_'.strtoupper($v))."</option>";
  }
  echo "</select><br/>";

  echo "<div id='cs_subtypes_caption' style='display:none;'>".NETCAT_CUSTOM_SUBTYPE.":</div>";
  echo "<div id='cs_subtypes' style=' margin-bottom: 10px;'></div>";
  echo "</div></fieldset>";

  foreach ( $extend_params  as $n => $tp ) {
    echo "<div id='extend_".$n."' class='cs_extends' style='display: none;'>";
           
    if ( !empty($tp) ) {
      echo "<fieldset><legend><b>".NETCAT_CUSTOM_ONCE_EXTEND."</b></legend><div style='width: 50%;'>";
      foreach ( $tp as $name => $v ) {
        if ( $v['type'] == 'checkbox' ) {
          echo nc_admin_checkbox($v['caption'], 'cs_'.$name, ( ($pst = $nc_core->input->fetch_get_post('cs_'.$name)) ? $pst : $settings[$name]) );
        }
        else if ( $v['type'] == 'text' ) {
          if ( is_array($settings[$name]) ) $settings[$name] = nc_a2f::array_to_string($settings[$name]);
          echo nc_admin_textarea($v['caption'], 'cs_'.$name, ( ($pst = $nc_core->input->fetch_get_post('cs_'.$name)) ? $pst : $settings[$name]), 1);
        }
        else if ( $v['type'] == 'classificator' ) {
          echo nc_admin_classificators($v['caption'], 'cs_'.$name, ( ($pst = $nc_core->input->fetch_get_post('cs_'.$name)) ? $pst : $settings[$name]), 'width:100%;');
        }
        else if ( $v['type'] == 'static' ) {
          echo nc_admin_select_static($v['caption'], 'cs_'.$name, $settings[$name]);
        }
        else {
          echo nc_admin_input($v['caption'], 'cs_'.$name, ( ($pst = $nc_core->input->fetch_get_post('cs_'.$name)) ? $pst : $settings[$name]), 0, 'width:100%;  margin-bottom: 10px;' );
        }
      }
      echo "</div></fieldset>";
    }
    echo "</div>";
  }
  echo nc_admin_js_resize();

  echo "</form>";

  echo "<script type='text/javascript'>
          nc_cs.changetype();
          nc_cs.changesubtype();
        </script>";
  $UI_CONFIG->actionButtons[] = array(
      "id" => "submit",
      "caption" => ( $param ? NETCAT_CUSTOM_ONCE_SAVE : NETCAT_CUSTOM_ONCE_ADD),
      "action" => "mainView.submitIframeForm()"
    );

  return 0;
}


/**
 * Сохранение информации о конкретном параметре пользовательских настроек
 * @return <type>
 */
function nc_customsettings_save_once ( ) {
  $nc_core = nc_Core::get_object();
  $ClassID = $nc_core->input->fetch_get_post('ClassID');
  $TemplateID = $nc_core->input->fetch_get_post('TemplateID');
  $old_name = $nc_core->input->fetch_get_post('param');
  $new_name = $nc_core->input->fetch_get_post('name');
  $type = $nc_core->input->fetch_get_post('type');
  $subtype = $nc_core->input->fetch_get_post('subtype');
  $caption = $nc_core->input->fetch_get_post('caption');
  $default_value = $nc_core->input->fetch_get_post('default_value');

  if ( !preg_match("/^[a-z0-9_]+$/i", $new_name) ) {
    throw new Exception(NETCAT_CUSTOM_ONCE_ERROR_FIELD_NAME);
  }
  if ( !$caption )  {
    throw new Exception(NETCAT_CUSTOM_ONCE_ERROR_CAPTION);
  }

  if ( $ClassID ) {
    $custom_settings = $nc_core->component->get_by_id( $ClassID , 'CustomSettingsTemplate');
  }
  else {
    $custom_settings = $nc_core->template->get_by_id( $TemplateID , 'CustomSettings');
  }

  if ( !$custom_settings ) {
    $custom_settings = array();
  }
  else {
    $a2f = new nc_a2f($custom_settings, '');
    $custom_settings = $a2f->eval_value($custom_settings);
  }
 
  $keys = empty($custom_settings) ? array() :  array_keys ($custom_settings);

  if ( in_array($new_name, $keys) && $new_name != $old_name ) {
    throw new Exception(NETCAT_CUSTOM_ONCE_ERROR_FIELD_EXISTS);
  }

  $param = array('type' => $type, 'subtype' => $subtype, 'caption' => $caption );
  if ( $default_value ) $param['default_value'] = $default_value;

  $input = $nc_core->input->fetch_get_post();

  // загружаем объект для работы с полем
  $classname = "nc_a2f_field_".$type.( $subtype ? "_".$subtype : "");
  if ( !class_exists($classname) ) return false;
  $fl = new $classname();
  $ex_params = $fl->get_extend_parametrs();
  if ( !empty($ex_params) ) foreach ( $ex_params as $k => $v ) {
    if ( $type == 'select' && $subtype == 'static' && $k == 'values') {
      $param[$k] = array();
      if ( !empty($_POST['select_static_key']) ) foreach ( $_POST['select_static_key'] as $key => $val ) {
        if ( !$_POST['select_static_value'][$key] || !$val  ) continue;
        $param[$k][$val] = $_POST['select_static_value'][$key];
      }
    }
    else {
      if ( isset($input['cs_'.$k]) ) $param[$k] = $input['cs_'.$k];
    }
  }

  /*
  foreach ( $input as $k => $v ) {
    if ( nc_preg_match('/cs_([a-z0-9_]+)/i', $k, $match) ) $param[$match[1]] = $v;
  }*/

  $custom_settings_new = array();
  if ( !empty($custom_settings) ) foreach ( $custom_settings as $k => $v ) {
    if ( $k != $old_name && $k != $new_name ) $custom_settings_new[$k] = $v;
    if ( $k == $old_name ||  $k == $new_name ) $custom_settings_new[$new_name] = $param;
  }

  if ( !$old_name ) {
    $custom_settings_new[$new_name] = $param;
  }
  

  $s = nc_a2f::array_to_string($custom_settings_new);
  $s = $custom_settings_new ? '$settings_array = '.$s.';' : '';

  if ( $ClassID ) {
    $nc_core->component->update($ClassID, array('CustomSettingsTemplate' => $s));
  }
  else {
    $nc_core->template->update($TemplateID, array('CustomSettings' => $s));
  }

  return true;
}


/**
 * Показ формы ручного редактирования пользовательских настроек
 * @param int номер компонента
 * @param int номер макета дизайна
 */
function nc_customsettings_show_manual (  $ClassID = 0, $TemplateID = 0 ) {
  global $UI_CONFIG;
  $nc_core = nc_Core::get_object();
  
  if ( $ClassID ) {
    $custom_settings = $nc_core->component->get_by_id( $ClassID , 'CustomSettingsTemplate');
  }
  else {
    $custom_settings = $nc_core->template->get_by_id( $TemplateID , 'CustomSettings');
  }

  echo "<form action='index.php' method='post' >";
  echo "<input type='hidden' name='phase' value='".( $ClassID ? "261" : "101")."' />";
  echo "<input type='hidden' name='ClassID' value='".$ClassID."' />";
  echo "<input type='hidden' name='TemplateID' value='".$TemplateID."' />";
  echo nc_admin_textarea(NETCAT_CUSTOM_USETTINGS, 'CustomSettings', $custom_settings, 1,0, "height:286px;");
  echo nc_admin_js_resize();
  echo "</form>";

  $UI_CONFIG->actionButtons[] = array(
    "id" => "back",
    "align" => "left",
    "caption" => CONTROL_AUTH_HTML_BACK,
    "location" => ($ClassID  ? "dataclass.custom(".$ClassID.")" : "template.custom(".$TemplateID.")")
  );

 $UI_CONFIG->actionButtons[] = array(
    "id" => "submit",
    "caption" => NETCAT_CUSTOM_ONCE_SAVE,
    "action" => "mainView.submitIframeForm()"
  );
}

class nc_admin_fieldset  {
  protected $name, $id, $mode, $text;
  public function  __construct( $name, $mode = 'on') {
    $this->name = htmlspecialchars($name);
    $this->mode = $mode;
    $this->id = md5($name.rand(0, 1000));
    $this->text = '';
  }

  public function add ( $text ) {
    $this->text .= "\n".$text;
  }

  public function result () {
    $res = "<div id='".$this->id."Off' style='cursor: pointer; display: none' onclick='this.style.display=\"none\"; document.getElementById(\"".$this->id."On\").style.display=\"\";'>\n";
    $res .= "\t<b> &#x25BA; ".$this->name."</b>\n";
    $res .= "</div>\n\n";

    $res .= "<div id='".$this->id."On'>\n<fieldset>";
    $res .= "<legend><b style='color:grey; cursor: pointer;' onclick='document.getElementById(\"".$this->id."On\").style.display=\"none\"; document.getElementById(\"".$this->id."Off\").style.display=\"\";'> &#x25BC; ".$this->name.":</b></legend>\n";

    $res .= $this->text;

    $res .= "</fieldset>\n</div><br/>\n";

    return $res;
  }
}
