<?php
/*$Id$*/
if ( !class_exists("nc_System") ) die("Unable to load file.");
/**
* Класс для изменения изображений
* 
* <p>Класс содержит статические методы для работы с изображениями:
* <b>imgResize</b> - созданием уменьшенной копии изображения</p>
* этот класс наследуется от  <b>nc_System</b>
*
* @see nc_ImageTransform::imgResize()
*/
class nc_ImageTransform extends nc_System {

  protected static $_db;

/**
 *
 * Создает уменьшенную копию изображения
 * @access public
 * @static 
 * @param string $src_img  Путь к исходному изображению
 * @param string $dest_img Путь к создаваемому изображению
 * @param int    $width    Ширина нового изображения
 * @param int    $height   Высота нового изображения
 * @param int    $mode     [optional] Режим уменьшения: 0 - пропорционально уменьшает; 1 - вписывает в указанные размеры, обрезая края
 * @param string $format   [optional] Формат создаваемого изображения (jpg, gif, png)
 * @param int    $quality  [optional] Качество сжатия изображения (0-100) при $format=='jpg'
 * @return mixed В случае ошибки возвратит false иначе возвратит путь к созданному файлу
 */
  public static function imgResize($src_img, $dest_img, $width, $height, $mode=0, $format='jpg', $quality=90, $message_id = 0, $field = 0) {

    global $classID, $systemTableID;
    $nc_core = nc_Core::get_object();

    if (!file_exists($src_img)) return false;
    $img_size = getimagesize($src_img);
    if ($img_size === false) return false;

    $img_format = strtolower(substr($img_size['mime'], strpos($img_size['mime'], '/') + 1));
    if (!function_exists($fn_imgcreatefrom = 'imagecreatefrom'.$img_format)) return false;

    $x_ratio = $width  / $img_size[0];
    $y_ratio = $height / $img_size[1];

    $fn_extrem = $mode ? 'max' : 'min';
    $ratio = $fn_extrem($x_ratio, $y_ratio);

    $new_width  = $x_ratio == $ratio ? $width  : floor($img_size[0] * $ratio);
    $new_height = $x_ratio != $ratio ? $height : floor($img_size[1] * $ratio);
    $new_left = $x_ratio == $ratio || !$mode ? 0 : floor(($width - $new_width) / 2);
    $new_top  = $x_ratio != $ratio || !$mode ? 0 : floor(($height - $new_height) / 2);

    $gd_dest_img = imagecreatetruecolor($mode?$width:$new_width, $mode?$height:$new_height);
    $gd_src_img = $fn_imgcreatefrom($src_img);

    imagecopyresampled($gd_dest_img, $gd_src_img, $new_left, $new_top, 0, 0, $new_width, $new_height, $img_size[0], $img_size[1]);
    switch ($format) {
      case 'gif': imagegif ($gd_dest_img, $dest_img); break;
      case 'png': imagepng ($gd_dest_img, $dest_img); break;
      default:    imagejpeg($gd_dest_img, $dest_img, $quality); break;
    }
    imagedestroy($gd_dest_img);
    imagedestroy($gd_src_img);

    // нужно поменять размер в таблице Filetable
    $HTTP_FILES_PATH_PREG = str_replace("/", "\/", $nc_core->HTTP_FILES_PATH);
    // есть файл в защищенной фс
    if (preg_match("/".$HTTP_FILES_PATH_PREG."([0-9uct]+)\/([0-9]+\/)?([0-9A-Z]{32})/i", $dest_img, $matches)) {

      $filename = $matches[3];
      $size = filesize($dest_img);
  
      $nc_core->db->query("UPDATE `Filetable`
                           SET `File_Size` = '".intval($size)."'
                           WHERE `Virt_Name` = '".$nc_core->db->escape($filename)."'  ");
    }


    // обновление таблицы MessageXX, User
    $message_id = intval($message_id);
    if ( $message_id && $field ) {
      // информация о поле
      $fld = $nc_core->db->get_row("
      	SELECT `Class_ID`, `System_Table_ID`, `Field_Name`
        FROM `Field`
        WHERE `Field_ID` = '".intval($field)."'
        OR (`Field_Name` = '".$nc_core->db->escape($field)."' AND ( `Class_ID` = '".intval($classID)."' OR `System_Table_ID` = '".intval($systemTableID)."'))
        LIMIT 1", ARRAY_A);

      // определение имени таблицы
      if ( $fld['Class_ID'] ) {
        $table = 'Message'.intval($fld['Class_ID']);
        $where = " `Message_ID` = '".$message_id."' ";
      }
      else {
        $table = $nc_core->db->get_var("SELECT `System_Table_Name` FROM `System_Table` WHERE `System_Table_ID` = '".intval($fld['System_Table_ID'])."' ");
        $where = " `".$table."_ID` = '".$message_id."'  ";

      }

      if ( !$table ) {
        return false;
      }
      // текущее значение поля объекта
      $value = $nc_core->db->get_var("SELECT `".$fld['Field_Name']."` FROM `".$table."` WHERE ".$where." ");

      if ( !$value ) {
        return false;
      }
      // обновляем размер
      $size = filesize($dest_img);
      $value = nc_preg_replace("/:[0-9]+/", ":".$size, preg_quote($value), 1);
      
      $nc_core->db->query("UPDATE `".$table."` SET `".$fld['Field_Name']."` = '".$nc_core->db->escape($value)."' WHERE ".$where." ");
    }

    return $dest_img;
  }
  
  /**
   * Функции для создания thumbnails для полей типа файл,
   * в действиях после добавления, после изменения.
   *
   * @global array  $GLOBALS
   * @global string $FILES_FOLDER
   * @param string  $src_field_name - имя поля-источника
   * @param string  $dest_field_name - имя поля-приёмника
   * @param int     $width       Ширина нового изображения
   * @param int     $height      Высота нового изображения
   * @param int     $mode     [optional] Режим уменьшения: 0 - пропорционально уменьшает; 1 - вписывает в указанные размеры, обрезая края
   * @param string  $format   [optional] Формат создаваемого изображения (jpg, gif, png)
   * @param int     $quality  [optional] Качество сжатия изображения (0-100) при $format=='jpg'
   *
   * @return bool true в случае удачи, false - в случае ошибки.
   * @access public
   * @static
   */

  public static function createThumb($src_field_name, $dest_field_name, $width, $height, $mode=0, $format='jpg', $quality=90) {

    global $GLOBALS;
    global $message, $classID;
    global $nc_core;

    $src_field_id  = $GLOBALS['fldID'][array_search($src_field_name,  $GLOBALS['fld'])];
    $is_sys = $nc_core->db->get_row("SELECT Class_ID, System_Table_ID from Field WHERE Field_ID = '".$src_field_id."'", ARRAY_A);
    $is_sys = $is_sys["System_Table_ID"];
    $dest_field_id = $GLOBALS['fldID'][array_search($dest_field_name, $GLOBALS['fld'])];

    return self::createThumb_byID($classID, $message, $src_field_id, $dest_field_id, $width, $height,$mode,$format,$quality, $src_field_name, $is_sys);
  }
  /**
   *  Функции для создания thumbnails для полей типа файл
   *
   *
   * @global  object $nc_core
   * @param   int   $classID - идентифиактор класса (компонента)
   * @param   int   $message - номер объекты
   * @param   int   $field_src_id - идентификатор поля источника
   * @param   int   $field_dst_id - идентификатор поля приемника
   * @param   int   $width    Ширина нового изображения
   * @param   int   $height   Высота нового изображения
   * @param   int   $mode     [optional] Режим уменьшения: 0 - пропорционально уменьшает; 1 - вписывает в указанные размеры, обрезая края
   * @param   string $format  [optional] Формат создаваемого изображения (jpg, gif, png)
   * @param   int   $quality  [optional] Качество сжатия изображения (0-100) при $format=='jpg'
   * @param   string $field_name_src
   * @return  bool true в случае удачи, false - в случае ошибки.
   * @access  public
   * @static
   */
  public static function createThumb_byID($classID,$message,$field_src_id, $field_dst_id, $width, $height, $mode=0, $format='jpg', $quality=90, $field_name_src='', $is_sys=false) {
    global $nc_core;
    
    // проверка аргументов
    $message += 0;
    $field_src_id += 0;
    $field_dst_id += 0;
    $classID += 0;
    if ( !$message || !$field_src_id || !$field_dst_id || !$classID ) return false;
    
    # поиск исходного файла
    // латинское имя поля
    if ( !$field_name_src ) { $field_name_src = $nc_core->db->get_var("SELECT `Field_Name` FROM `Field` WHERE `Field_ID` = '".$field_src_id."'"); }
    if ( !$field_name_src ) return 0;

    // Значение поля в таблице объектов
    $message_field = ($is_sys ? $nc_core->db->get_row("SELECT * FROM `User` WHERE `User_ID` = '".$message."'", ARRAY_A) : $nc_core->db->get_row("SELECT * FROM `Message".$classID."` WHERE `Message_ID` = '".$message."'", ARRAY_A));
    $file_data = explode(':',  $message_field[$field_name_src]);
    $file_name = $file_data[0];
    $file_type = $file_data[1];
    $file_size = $file_data[2];
    $ext = substr($file_name, strrpos($file_name, ".")); // расширение файла
    $file_name = substr($file_name, 0, strrpos($file_name, ".")); // имя файла без расширения.
    // если ли файл в Filetable ?
    $filetable = $nc_core->db->get_row("SELECT * FROM `Filetable`
                                            WHERE `Message_ID` = '".intval($message)."' AND `Field_ID` = '".intval($field_src_id)."'", ARRAY_A);
    // определения полного пути к файлу
    if ( $filetable ) { // исходный файл в protected
      $path_src = rtrim($nc_core->FILES_FOLDER, '/').$filetable['File_Path'].$filetable['Virt_Name'];
    }
    else {
      if ( $file_data[3] ) { // orignal
        $path_src = $nc_core->FILES_FOLDER.$file_data[3];
      }
      else { // simple
        $path_src = $nc_core->FILES_FOLDER.$field_src_id."_".$message.$ext;
      }
    }

    // получение информации о поле-приемника
    $field_info_desc = $nc_core->db->get_row("SELECT `Field_Name`, `Format` FROM `Field` WHERE `Field_ID` = '".$field_dst_id."'", ARRAY_A);
    if ( !$field_info_desc) return false;

    //удаление старого файла
    require_once($nc_core->INCLUDE_FOLDER."s_files.inc.php");
    DeleteFile($field_dst_id, $field_info_desc['Field_Name'], $classID, 0, $message);

    // определение типа фс применика
    $fs = nc_field_parse_format($field_info_desc['Format'], NC_FIELDTYPE_FILE);
    $fs = $fs['fs'];

    $file_name.="_thumb";
    // определние имени файла на диске и диретории

    switch ( $fs ) {
      case NC_FS_PROTECTED:
        $path_dsc = ($is_sys ? 'u/' : $message_field['Subdivision_ID'].'/'.$message_field['Sub_Class_ID'].'/');
        $name_dsc =  md5($file_name.date("H:i:s d.m.Y").uniqid("netcat"));
        break;
      case NC_FS_ORIGINAL:
        $path_dsc = $message_field['Subdivision_ID'].'/'.$message_field['Sub_Class_ID'].'/';
        $name_dsc = nc_get_filename_for_original_fs($file_name.".".$format, $nc_core->FILES_FOLDER.$path_dsc);
        break;
      case NC_FS_SIMPLE:
        $path_dsc = '';
        $name_dsc = $field_dst_id."_".$message.".".$format;
        break;
    }

    // копирование файла
    //copy($path_src, $nc_core->FILES_FOLDER.$path_dsc.$name_dsc);
    // создание thumb
    $path_dest = ($is_sys ? $nc_core->FILES_FOLDER.'/u/'.$name_dsc : $nc_core->FILES_FOLDER.$path_dsc.$name_dsc);
    self::imgResize($path_src, $path_dest, $width, $height, $mode, $format, $quality);

    switch ( $fs ) {
      case NC_FS_PROTECTED :
        $img_size = getimagesize($path_dest);
        if ($img_size === false) {
          if ( file_exists($path_dest) ) unlink($path_dest);
          return false;
        }
        $dst_File_Type = $img_size['mime'];
        $dst_File_Size = filesize($path_dest);
        $insert_filetable_sql = "INSERT INTO `Filetable`(Real_Name, Virt_Name, File_Path, File_Type, File_Size, Message_ID, Field_ID)
                               VALUES('".$nc_core->db->escape($file_name).".".$format."', '".$nc_core->db->escape($name_dsc)."', '/".$nc_core->db->escape($path_dsc)."','".$dst_File_Type."', '".$dst_File_Size."', '".$message."', '".$field_dst_id."')";
        $nc_core->db->query($insert_filetable_sql);
        
        $in_db = $file_name.":".$dst_File_Type.":".$dst_File_Size; // то, что запишится в БД
        break;
      case NC_FS_ORIGINAL :
        $in_db = $file_name.".".$format.":".$format.":".$file_size; // то, что запишится в БД
        $in_db .= ":".($is_sys ? 'u/'.$name_dsc : $path_dsc.$name_dsc);
        break;
      default :
        $in_db = $file_name.".".$format.":".$format.":".$file_size; // то, что запишится в БД
        break;
    }
    // обновление инфы в БД
    ($is_sys ?  $nc_core->db->query("UPDATE `User` SET `".$field_info_desc['Field_Name']."` = '".$nc_core->db->escape($in_db)."' WHERE `User_ID` = '".$message."'"): $nc_core->db->query("UPDATE `Message".$classID."` SET `".$field_info_desc['Field_Name']."` = '".$nc_core->db->escape($in_db)."' WHERE `Message_ID` = '".$message."'"));
    
    return true;
  }

  public static function putWatermark_file ( $filepath, $watermark, $mode = 0 ) {
    $nc_core = nc_Core::get_object();
    // исходный файл
    if (!file_exists($filepath ) ) {
      $filepath = $nc_core->DOCUMENT_ROOT.$nc_core->SUB_FOLDER.$filepath;
    }
    if ( !file_exists($filepath) ) {
      trigger_error("File ".$filepath." not found.",  E_USER_WARNING);
      return;
    }
    $src = getimagesize($filepath);
    $src_w = $src[0]; // ширина
    $src_h = $src[1]; // высота
    $src_type = strtolower(substr($src['mime'], strpos($src['mime'], '/')+1)); // тип
    // в зависимости от типа - разные функции
    $func = function_exists("imagecreatefrom".$src_type) ? "imagecreatefrom".$src_type : "imagecreatefromjpeg";
    // ресурс
    $img_src = $func($filepath);

    // ватермарк
    if (!file_exists($watermark ) ) {
      $filepath = $nc_core->DOCUMENT_ROOT.$nc_core->SUB_FOLDER.$watermark;
    }
    if ( !file_exists($watermark) ) {
      trigger_error("File ".$watermark." not found.",  E_USER_WARNING);
      return;
    }
    $water = getimagesize($watermark);
    $water_w = $water[0]; // ширина
    $water_h = $water[1]; // высота
    $water_type = strtolower(substr($water['mime'], strpos($water['mime'], '/')+1)); // тип
    // в зависимости от типа - разные функции
    $func = function_exists("imagecreatefrom".$water_type) ? "imagecreatefrom".$water_type : "imagecreatefromjpeg";
    // ресурс
    $img_water = $func($watermark);

    // результат
    $img = imagecreatetruecolor($src_w, $src_h);
    // копируем в результат исходное изображение
    imagecopyresampled($img, $img_src, 0, 0, 0, 0, $src_w, $src_h, $src_w, $src_h);
    // опрделяем, куда копировать ватермарк
    switch ( $mode ) {
      case 1 : // левый верхний угол
        $x = $y = 3; // c небольшим сдвигом
        break;
      case 2: // правый верхний угол
        $x = $src_w - $water_w - 3;
        $y = 3;
        break;
      case 3: // левый низ
        $y = $src_h - $water_h - 3;
        $x = 3;
        break;
      case 4: // правый низ
        $x = $src_w - $water_w - 3;
        $y = $src_h - $water_h - 3;
        break;
      default: // по центру
        $x = floor(($src_w - $water_w) / 2);
				$y = floor(($src_h - $water_h) / 2);
    }
    if ( $x < 0 ) $x = 0;
    if ( $y < 0 ) $y = 0;
    // копируем  ватермарк
    imagecopy($img, $img_water, $x, $y, 0, 0, $water_w, $water_h);

    //записываем в файл
    $func = function_exists("image".$src_type) ? "image".$src_type : "imagejpg";
    // можно задать качество
    if ( func == "imagejpeg" || $func == "imagepng") {
      $r =  $func($img, $filepath, 95);
    }
    else {
      $r =  $func($img, $filepath);
    }

    imagedestroy($img);
    imagedestroy($img_src);
    imagedestroy($img_water);
    
    return $r;
  }


  public static function putWatermark ( $classID, $field, $message, $watermark, $mode = 0 ) {
    global $nc_core, $db;
    $message = intval($message);
    
    $src = nc_file_path($classID, $message, $field);
    if ( !$src ) return false;
    
    // вставляем ватермарк
    self::putWatermark_file($src, $watermark, $mode);

    // теперь нужно обновить размер
    
    $systemTableID = 0;
    $system_tables = array("Catalogue" => 1,"Subdivision" => 2, "User" => 3, "Template" => 4);

    // определяем таблицу и первичный ключ в ней
    if ( !is_int($classID) ) {
      $table = $db->escape($classID);
      $pk = $db->escape($classID)."_ID";
      $systemTableID = $system_tables[$classID];
    }
    else {
      $table = "Message".intval($classID);
      $pk = "Message_ID";
    }
    
    // определяем номер поля и его имя
    if ( is_int($field ) ) {
      $field_id = intval($field);
      $field_name = $db->get_var("SELECT `Field_Name` FROM `Field` WHERE `Field_ID` = '".$field_id."' ");
    }
    else {
      $field_name = $db->escape($field);
      $field_id = $db->get_var("SELECT `Field_ID` FROM `Field` WHERE `Field_Name` = '".$field_name."' AND ".( $systemTableID ? "`System_Table_ID` = '".$systemTableID."'" : "`Class_ID` = '".$classID."' ")." ");
    }
    // новое значение
    clearstatcache();
    $filesize = filesize($nc_core->DOCUMENT_ROOT.$src);

    $old_value = $db->get_var("SELECT `".$field_name."` FROM `".$table."` WHERE `".$pk."` = '".$message."' ");
    $new_value = preg_replace("/:(\d+):/", ':'.$filesize.':', $old_value);
    $new_value = preg_replace("/:(\d+)$/", ':'.$filesize, $new_value);

    $db->query("UPDATE `".$table."` SET `".$field_name."` = '".$db->escape($new_value)."' WHERE `".$pk."` = '".$message."' ");
    // и в таблице Filetable
    $db->query("UPDATE `Filetable` SET `File_Size` = '".$filesize."' WHERE `Message_ID` = '".$message."' AND `Field_ID` = '".$field_id."' ");

   


    



  }
}
?>
